from typing import Any, Dict, Type, TypeVar

import attr

from ..models.campaign_user_session_state import CampaignUserSessionState

T = TypeVar("T", bound="CampaignUserSession")


@attr.s(auto_attribs=True)
class CampaignUserSession:
    """
    Attributes:
        id (str):
        session_id (str):
        session_mode_id (str):
        state (CampaignUserSessionState):
        username (str):
    """

    id: str
    session_id: str
    session_mode_id: str
    state: CampaignUserSessionState
    username: str

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        session_id = self.session_id
        session_mode_id = self.session_mode_id
        state = self.state.value

        username = self.username

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "sessionId": session_id,
                "sessionModeId": session_mode_id,
                "state": state,
                "username": username,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        session_id = d.pop("sessionId")

        session_mode_id = d.pop("sessionModeId")

        state = CampaignUserSessionState(d.pop("state"))

        username = d.pop("username")

        campaign_user_session = cls(
            id=id,
            session_id=session_id,
            session_mode_id=session_mode_id,
            state=state,
            username=username,
        )

        return campaign_user_session



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="MetadataDefinitionEntry")


@attr.s(auto_attribs=True)
class MetadataDefinitionEntry:
    """
    Attributes:
        distinct_metadata_values (List[str]):
        is_editable (bool):
        is_multiple (bool):
        metadata_name (str):
    """

    distinct_metadata_values: List[str]
    is_editable: bool
    is_multiple: bool
    metadata_name: str

    def to_dict(self) -> Dict[str, Any]:
        distinct_metadata_values = self.distinct_metadata_values

        is_editable = self.is_editable
        is_multiple = self.is_multiple
        metadata_name = self.metadata_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "distinctMetadataValues": distinct_metadata_values,
                "isEditable": is_editable,
                "isMultiple": is_multiple,
                "metadataName": metadata_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        distinct_metadata_values = cast(List[str], d.pop("distinctMetadataValues"))

        is_editable = d.pop("isEditable")

        is_multiple = d.pop("isMultiple")

        metadata_name = d.pop("metadataName")

        metadata_definition_entry = cls(
            distinct_metadata_values=distinct_metadata_values,
            is_editable=is_editable,
            is_multiple=is_multiple,
            metadata_name=metadata_name,
        )

        return metadata_definition_entry



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.get_segment_context_context_output import GetSegmentContextContextOutput
from ...models.segment_contexts import SegmentContexts
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/segments/_context".format(client.base_url, projectName=project_name)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["documentIdentifier"] = document_identifier

    params["segmentStart"] = segment_start

    params["fromBefore"] = from_before

    params["sizeBefore"] = size_before

    params["fromAfter"] = from_after

    params["sizeAfter"] = size_after

    json_context_output: Union[Unset, None, str] = UNSET
    if not isinstance(context_output, Unset):
        json_context_output = context_output.value if context_output else None

    params["contextOutput"] = json_context_output

    params["includeAnnotations"] = include_annotations

    params["htmlVersion"] = html_version

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[SegmentContexts]:
    if response.status_code == HTTPStatus.OK:
        response_200 = SegmentContexts.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[SegmentContexts]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Response[SegmentContexts]:
    """Get segments surrounding a segment

    Args:
        project_name (str):
        document_identifier (str):
        segment_start (int):
        from_before (Union[Unset, None, int]):
        size_before (Union[Unset, None, int]):  Default: 1.
        from_after (Union[Unset, None, int]):
        size_after (Union[Unset, None, int]):  Default: 1.
        context_output (Union[Unset, None, GetSegmentContextContextOutput]):  Default:
            GetSegmentContextContextOutput.SEGMENTS.
        include_annotations (Union[Unset, None, bool]):
        html_version (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SegmentContexts]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        document_identifier=document_identifier,
        segment_start=segment_start,
        from_before=from_before,
        size_before=size_before,
        from_after=from_after,
        size_after=size_after,
        context_output=context_output,
        include_annotations=include_annotations,
        html_version=html_version,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Optional[SegmentContexts]:
    """Get segments surrounding a segment

    Args:
        project_name (str):
        document_identifier (str):
        segment_start (int):
        from_before (Union[Unset, None, int]):
        size_before (Union[Unset, None, int]):  Default: 1.
        from_after (Union[Unset, None, int]):
        size_after (Union[Unset, None, int]):  Default: 1.
        context_output (Union[Unset, None, GetSegmentContextContextOutput]):  Default:
            GetSegmentContextContextOutput.SEGMENTS.
        include_annotations (Union[Unset, None, bool]):
        html_version (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SegmentContexts]
    """

    return sync_detailed(
        project_name=project_name,
        client=client,
        document_identifier=document_identifier,
        segment_start=segment_start,
        from_before=from_before,
        size_before=size_before,
        from_after=from_after,
        size_after=size_after,
        context_output=context_output,
        include_annotations=include_annotations,
        html_version=html_version,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Response[SegmentContexts]:
    """Get segments surrounding a segment

    Args:
        project_name (str):
        document_identifier (str):
        segment_start (int):
        from_before (Union[Unset, None, int]):
        size_before (Union[Unset, None, int]):  Default: 1.
        from_after (Union[Unset, None, int]):
        size_after (Union[Unset, None, int]):  Default: 1.
        context_output (Union[Unset, None, GetSegmentContextContextOutput]):  Default:
            GetSegmentContextContextOutput.SEGMENTS.
        include_annotations (Union[Unset, None, bool]):
        html_version (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SegmentContexts]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        document_identifier=document_identifier,
        segment_start=segment_start,
        from_before=from_before,
        size_before=size_before,
        from_after=from_after,
        size_after=size_after,
        context_output=context_output,
        include_annotations=include_annotations,
        html_version=html_version,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Optional[SegmentContexts]:
    """Get segments surrounding a segment

    Args:
        project_name (str):
        document_identifier (str):
        segment_start (int):
        from_before (Union[Unset, None, int]):
        size_before (Union[Unset, None, int]):  Default: 1.
        from_after (Union[Unset, None, int]):
        size_after (Union[Unset, None, int]):  Default: 1.
        context_output (Union[Unset, None, GetSegmentContextContextOutput]):  Default:
            GetSegmentContextContextOutput.SEGMENTS.
        include_annotations (Union[Unset, None, bool]):
        html_version (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SegmentContexts]
    """

    return (
        await asyncio_detailed(
            project_name=project_name,
            client=client,
            document_identifier=document_identifier,
            segment_start=segment_start,
            from_before=from_before,
            size_before=size_before,
            from_after=from_after,
            size_after=size_after,
            context_output=context_output,
            include_annotations=include_annotations,
            html_version=html_version,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.category_creation_mode import CategoryCreationMode
from ..models.category_status import CategoryStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="Category")


@attr.s(auto_attribs=True)
class Category:
    """A document category

    Attributes:
        document_identifier (str): identifier of the document
        label_name (str): The label name
        creation_mode (Union[Unset, CategoryCreationMode]): Creation mode
        score (Union[Unset, float]): Score of the category
        status (Union[Unset, CategoryStatus]): Status of the category
    """

    document_identifier: str
    label_name: str
    creation_mode: Union[Unset, CategoryCreationMode] = UNSET
    score: Union[Unset, float] = UNSET
    status: Union[Unset, CategoryStatus] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        document_identifier = self.document_identifier
        label_name = self.label_name
        creation_mode: Union[Unset, str] = UNSET
        if not isinstance(self.creation_mode, Unset):
            creation_mode = self.creation_mode.value

        score = self.score
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "documentIdentifier": document_identifier,
                "labelName": label_name,
            }
        )
        if creation_mode is not UNSET:
            field_dict["creationMode"] = creation_mode
        if score is not UNSET:
            field_dict["score"] = score
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        document_identifier = d.pop("documentIdentifier")

        label_name = d.pop("labelName")

        _creation_mode = d.pop("creationMode", UNSET)
        creation_mode: Union[Unset, CategoryCreationMode]
        if isinstance(_creation_mode, Unset):
            creation_mode = UNSET
        else:
            creation_mode = CategoryCreationMode(_creation_mode)

        score = d.pop("score", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, CategoryStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = CategoryStatus(_status)

        category = cls(
            document_identifier=document_identifier,
            label_name=label_name,
            creation_mode=creation_mode,
            score=score,
            status=status,
        )

        return category



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="InputLabel")


@attr.s(auto_attribs=True)
class InputLabel:
    """
    Attributes:
        label (str):
    """

    label: str

    def to_dict(self) -> Dict[str, Any]:
        label = self.label

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "label": label,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label")

        input_label = cls(
            label=label,
        )

        return input_label



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="BearerToken")


@attr.s(auto_attribs=True)
class BearerToken:
    """
    Attributes:
        access_token (str):
        username (str):
    """

    access_token: str
    username: str

    def to_dict(self) -> Dict[str, Any]:
        access_token = self.access_token
        username = self.username

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "access_token": access_token,
                "username": username,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        access_token = d.pop("access_token")

        username = d.pop("username")

        bearer_token = cls(
            access_token=access_token,
            username=username,
        )

        return bearer_token



""" Contains some shared types for properties """
from http import HTTPStatus
from typing import BinaryIO, Generic, MutableMapping, Optional, Tuple, TypeVar

import attr
from httpx import HTTPError, codes


class Unset:
    def __bool__(self) -> bool:
        return False


UNSET: Unset = Unset()

FileJsonType = Tuple[Optional[str], BinaryIO, Optional[str]]


@attr.s(auto_attribs=True)
class File:
    """Contains information for file uploads"""

    payload: BinaryIO
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> FileJsonType:
        """Return a tuple representation that httpx will accept for multipart/form-data"""
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


@attr.s(auto_attribs=True)
class Response(Generic[T]):
    """A response from an endpoint"""

    status_code: HTTPStatus
    content: bytes
    headers: MutableMapping[str, str]
    parsed: Optional[T]

    @property
    def is_informational(self) -> bool:
        """
        A property which is `True` for 1xx status codes, `False` otherwise.
        """
        return codes.is_informational(int(self.status_code))

    @property
    def is_success(self) -> bool:
        """
        A property which is `True` for 2xx status codes, `False` otherwise.
        """
        return codes.is_success(int(self.status_code))

    @property
    def is_redirect(self) -> bool:
        """
        A property which is `True` for 3xx status codes, `False` otherwise.

        Note that not all responses with a 3xx status code indicate a URL redirect.

        Use `response.has_redirect_location` to determine responses with a properly
        formed URL redirection.
        """
        return codes.is_redirect(int(self.status_code))

    @property
    def is_client_error(self) -> bool:
        """
        A property which is `True` for 4xx status codes, `False` otherwise.
        """
        return codes.is_client_error(int(self.status_code))

    @property
    def is_server_error(self) -> bool:
        """
        A property which is `True` for 5xx status codes, `False` otherwise.
        """
        return codes.is_server_error(int(self.status_code))

    @property
    def is_error(self) -> bool:
        """
        A property which is `True` for 4xx and 5xx status codes, `False` otherwise.
        """
        return codes.is_error(int(self.status_code))

    @property
    def has_redirect_location(self) -> bool:
        """
        Returns True for 3xx responses with a properly formed URL redirection,
        `False` otherwise.
        """
        return (
            self.status_code
            in (
                # 301 (Cacheable redirect. Method may change to GET.)
                codes.MOVED_PERMANENTLY,
                # 302 (Uncacheable redirect. Method may change to GET.)
                codes.FOUND,
                # 303 (Client should make a GET or HEAD request.)
                codes.SEE_OTHER,
                # 307 (Equiv. 302, but retain method)
                codes.TEMPORARY_REDIRECT,
                # 308 (Equiv. 301, but retain method)
                codes.PERMANENT_REDIRECT,
            )
            and "Location" in self.headers
        )

    def raise_for_status(self):
        """Raises :class:`HTTPError`, if one occurred."""

        http_error_msg = ""
        if isinstance(self.content, bytes):
            # We attempt to decode utf-8 first because some servers
            # choose to localize their reason strings. If the string
            # isn't utf-8, we fall back to iso-8859-1 for all other
            # encodings. (See PR #3538)
            try:
                reason = self.content.decode("utf-8")
            except UnicodeDecodeError:
                reason = self.content.decode("iso-8859-1")
        else:
            reason = self.content

        if 400 <= int(self.status_code) < 500:
            http_error_msg = "%s Client Error: %s" % (self.status_code, reason)

        elif 500 <= int(self.status_code) < 600:
            http_error_msg = "%s Server Error: %s" % (self.status_code, reason)

        if http_error_msg:
            raise HTTPError(http_error_msg)


__all__ = ["File", "Response", "FileJsonType"]



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.classification_config import ClassificationConfig
    from ..models.project_open_session import ProjectOpenSession


T = TypeVar("T", bound="ProjectBean")


@attr.s(auto_attribs=True)
class ProjectBean:
    """
    Attributes:
        description (str):
        image (str):
        label (str):
        lang (str):
        name (str):
        algorithms (Union[Unset, List[str]]):
        annotations (Union[Unset, int]):
        categories (Union[Unset, int]):
        classification (Union[Unset, ClassificationConfig]):
        components (Union[Unset, List[str]]):
        created_by (Union[Unset, str]):
        created_date (Union[Unset, str]):
        dev_patches (Union[Unset, List[str]]):
        documents (Union[Unset, int]):
        engines (Union[Unset, List[str]]):
        group_name (Union[Unset, str]):
        has_split (Union[Unset, bool]):
        metafacets (Union[Unset, List[Any]]):
        nature (Union[Unset, str]):
        open_session (Union[Unset, ProjectOpenSession]):
        owner (Union[Unset, str]):
        private (Union[Unset, bool]):
        read_only (Union[Unset, bool]):
        segments (Union[Unset, int]):
        shared (Union[Unset, bool]):
        terms (Union[Unset, int]):
        version (Union[Unset, str]):
    """

    description: str
    image: str
    label: str
    lang: str
    name: str
    algorithms: Union[Unset, List[str]] = UNSET
    annotations: Union[Unset, int] = UNSET
    categories: Union[Unset, int] = UNSET
    classification: Union[Unset, "ClassificationConfig"] = UNSET
    components: Union[Unset, List[str]] = UNSET
    created_by: Union[Unset, str] = UNSET
    created_date: Union[Unset, str] = UNSET
    dev_patches: Union[Unset, List[str]] = UNSET
    documents: Union[Unset, int] = UNSET
    engines: Union[Unset, List[str]] = UNSET
    group_name: Union[Unset, str] = UNSET
    has_split: Union[Unset, bool] = UNSET
    metafacets: Union[Unset, List[Any]] = UNSET
    nature: Union[Unset, str] = UNSET
    open_session: Union[Unset, "ProjectOpenSession"] = UNSET
    owner: Union[Unset, str] = UNSET
    private: Union[Unset, bool] = UNSET
    read_only: Union[Unset, bool] = UNSET
    segments: Union[Unset, int] = UNSET
    shared: Union[Unset, bool] = UNSET
    terms: Union[Unset, int] = UNSET
    version: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        description = self.description
        image = self.image
        label = self.label
        lang = self.lang
        name = self.name
        algorithms: Union[Unset, List[str]] = UNSET
        if not isinstance(self.algorithms, Unset):
            algorithms = self.algorithms

        annotations = self.annotations
        categories = self.categories
        classification: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.classification, Unset):
            classification = self.classification.to_dict()

        components: Union[Unset, List[str]] = UNSET
        if not isinstance(self.components, Unset):
            components = self.components

        created_by = self.created_by
        created_date = self.created_date
        dev_patches: Union[Unset, List[str]] = UNSET
        if not isinstance(self.dev_patches, Unset):
            dev_patches = self.dev_patches

        documents = self.documents
        engines: Union[Unset, List[str]] = UNSET
        if not isinstance(self.engines, Unset):
            engines = self.engines

        group_name = self.group_name
        has_split = self.has_split
        metafacets: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.metafacets, Unset):
            metafacets = self.metafacets

        nature = self.nature
        open_session: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.open_session, Unset):
            open_session = self.open_session.to_dict()

        owner = self.owner
        private = self.private
        read_only = self.read_only
        segments = self.segments
        shared = self.shared
        terms = self.terms
        version = self.version

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "description": description,
                "image": image,
                "label": label,
                "lang": lang,
                "name": name,
            }
        )
        if algorithms is not UNSET:
            field_dict["algorithms"] = algorithms
        if annotations is not UNSET:
            field_dict["annotations"] = annotations
        if categories is not UNSET:
            field_dict["categories"] = categories
        if classification is not UNSET:
            field_dict["classification"] = classification
        if components is not UNSET:
            field_dict["components"] = components
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if created_date is not UNSET:
            field_dict["createdDate"] = created_date
        if dev_patches is not UNSET:
            field_dict["devPatches"] = dev_patches
        if documents is not UNSET:
            field_dict["documents"] = documents
        if engines is not UNSET:
            field_dict["engines"] = engines
        if group_name is not UNSET:
            field_dict["groupName"] = group_name
        if has_split is not UNSET:
            field_dict["hasSplit"] = has_split
        if metafacets is not UNSET:
            field_dict["metafacets"] = metafacets
        if nature is not UNSET:
            field_dict["nature"] = nature
        if open_session is not UNSET:
            field_dict["openSession"] = open_session
        if owner is not UNSET:
            field_dict["owner"] = owner
        if private is not UNSET:
            field_dict["private"] = private
        if read_only is not UNSET:
            field_dict["readOnly"] = read_only
        if segments is not UNSET:
            field_dict["segments"] = segments
        if shared is not UNSET:
            field_dict["shared"] = shared
        if terms is not UNSET:
            field_dict["terms"] = terms
        if version is not UNSET:
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.classification_config import ClassificationConfig
        from ..models.project_open_session import ProjectOpenSession

        d = src_dict.copy()
        description = d.pop("description")

        image = d.pop("image")

        label = d.pop("label")

        lang = d.pop("lang")

        name = d.pop("name")

        algorithms = cast(List[str], d.pop("algorithms", UNSET))

        annotations = d.pop("annotations", UNSET)

        categories = d.pop("categories", UNSET)

        _classification = d.pop("classification", UNSET)
        classification: Union[Unset, ClassificationConfig]
        if isinstance(_classification, Unset):
            classification = UNSET
        else:
            classification = ClassificationConfig.from_dict(_classification)

        components = cast(List[str], d.pop("components", UNSET))

        created_by = d.pop("createdBy", UNSET)

        created_date = d.pop("createdDate", UNSET)

        dev_patches = cast(List[str], d.pop("devPatches", UNSET))

        documents = d.pop("documents", UNSET)

        engines = cast(List[str], d.pop("engines", UNSET))

        group_name = d.pop("groupName", UNSET)

        has_split = d.pop("hasSplit", UNSET)

        metafacets = cast(List[Any], d.pop("metafacets", UNSET))

        nature = d.pop("nature", UNSET)

        _open_session = d.pop("openSession", UNSET)
        open_session: Union[Unset, ProjectOpenSession]
        if isinstance(_open_session, Unset):
            open_session = UNSET
        else:
            open_session = ProjectOpenSession.from_dict(_open_session)

        owner = d.pop("owner", UNSET)

        private = d.pop("private", UNSET)

        read_only = d.pop("readOnly", UNSET)

        segments = d.pop("segments", UNSET)

        shared = d.pop("shared", UNSET)

        terms = d.pop("terms", UNSET)

        version = d.pop("version", UNSET)

        project_bean = cls(
            description=description,
            image=image,
            label=label,
            lang=lang,
            name=name,
            algorithms=algorithms,
            annotations=annotations,
            categories=categories,
            classification=classification,
            components=components,
            created_by=created_by,
            created_date=created_date,
            dev_patches=dev_patches,
            documents=documents,
            engines=engines,
            group_name=group_name,
            has_split=has_split,
            metafacets=metafacets,
            nature=nature,
            open_session=open_session,
            owner=owner,
            private=private,
            read_only=read_only,
            segments=segments,
            shared=shared,
            terms=terms,
            version=version,
        )

        return project_bean



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.sherpa_job_bean_status import SherpaJobBeanStatus
from ..models.sherpa_job_bean_type import SherpaJobBeanType
from ..types import UNSET, Unset

T = TypeVar("T", bound="SherpaJobBean")


@attr.s(auto_attribs=True)
class SherpaJobBean:
    """
    Attributes:
        created_at (int):
        created_by (str):
        current_step_count (int):
        description (str):
        id (str):
        project (str):
        project_label (str):
        status (SherpaJobBeanStatus):
        total_step_count (int):
        type (SherpaJobBeanType):
        upload_ids (List[str]):
        completed_at (Union[Unset, int]):
        status_message (Union[Unset, str]):
    """

    created_at: int
    created_by: str
    current_step_count: int
    description: str
    id: str
    project: str
    project_label: str
    status: SherpaJobBeanStatus
    total_step_count: int
    type: SherpaJobBeanType
    upload_ids: List[str]
    completed_at: Union[Unset, int] = UNSET
    status_message: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        created_at = self.created_at
        created_by = self.created_by
        current_step_count = self.current_step_count
        description = self.description
        id = self.id
        project = self.project
        project_label = self.project_label
        status = self.status.value

        total_step_count = self.total_step_count
        type = self.type.value

        upload_ids = self.upload_ids

        completed_at = self.completed_at
        status_message = self.status_message

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "createdAt": created_at,
                "createdBy": created_by,
                "currentStepCount": current_step_count,
                "description": description,
                "id": id,
                "project": project,
                "projectLabel": project_label,
                "status": status,
                "totalStepCount": total_step_count,
                "type": type,
                "uploadIds": upload_ids,
            }
        )
        if completed_at is not UNSET:
            field_dict["completedAt"] = completed_at
        if status_message is not UNSET:
            field_dict["statusMessage"] = status_message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        created_at = d.pop("createdAt")

        created_by = d.pop("createdBy")

        current_step_count = d.pop("currentStepCount")

        description = d.pop("description")

        id = d.pop("id")

        project = d.pop("project")

        project_label = d.pop("projectLabel")

        status = SherpaJobBeanStatus(d.pop("status"))

        total_step_count = d.pop("totalStepCount")

        type = SherpaJobBeanType(d.pop("type"))

        upload_ids = cast(List[str], d.pop("uploadIds"))

        completed_at = d.pop("completedAt", UNSET)

        status_message = d.pop("statusMessage", UNSET)

        sherpa_job_bean = cls(
            created_at=created_at,
            created_by=created_by,
            current_step_count=current_step_count,
            description=description,
            id=id,
            project=project,
            project_label=project_label,
            status=status,
            total_step_count=total_step_count,
            type=type,
            upload_ids=upload_ids,
            completed_at=completed_at,
            status_message=status_message,
        )

        return sherpa_job_bean



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PartialLexicon")


@attr.s(auto_attribs=True)
class PartialLexicon:
    """
    Attributes:
        label (str):
        color (Union[Unset, str]):
        name (Union[Unset, str]):
    """

    label: str
    color: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        color = self.color
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "label": label,
            }
        )
        if color is not UNSET:
            field_dict["color"] = color
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label")

        color = d.pop("color", UNSET)

        name = d.pop("name", UNSET)

        partial_lexicon = cls(
            label=label,
            color=color,
            name=name,
        )

        return partial_lexicon



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GroupPatch")


@attr.s(auto_attribs=True)
class GroupPatch:
    """
    Attributes:
        attached_roles (Union[Unset, List[str]]):
        label (Union[Unset, str]):
        max_users (Union[Unset, int]):
    """

    attached_roles: Union[Unset, List[str]] = UNSET
    label: Union[Unset, str] = UNSET
    max_users: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        attached_roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.attached_roles, Unset):
            attached_roles = self.attached_roles

        label = self.label
        max_users = self.max_users

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if attached_roles is not UNSET:
            field_dict["attachedRoles"] = attached_roles
        if label is not UNSET:
            field_dict["label"] = label
        if max_users is not UNSET:
            field_dict["maxUsers"] = max_users

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        attached_roles = cast(List[str], d.pop("attachedRoles", UNSET))

        label = d.pop("label", UNSET)

        max_users = d.pop("maxUsers", UNSET)

        group_patch = cls(
            attached_roles=attached_roles,
            label=label,
            max_users=max_users,
        )

        return group_patch



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="CampaignRolesChange")


@attr.s(auto_attribs=True)
class CampaignRolesChange:
    """
    Attributes:
        added_roles (List[str]):
        remove_current_funtional_roles (bool):
        removed_roles (List[str]):
        username (str):
    """

    added_roles: List[str]
    remove_current_funtional_roles: bool
    removed_roles: List[str]
    username: str

    def to_dict(self) -> Dict[str, Any]:
        added_roles = self.added_roles

        remove_current_funtional_roles = self.remove_current_funtional_roles
        removed_roles = self.removed_roles

        username = self.username

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "addedRoles": added_roles,
                "removeCurrentFuntionalRoles": remove_current_funtional_roles,
                "removedRoles": removed_roles,
                "username": username,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        added_roles = cast(List[str], d.pop("addedRoles"))

        remove_current_funtional_roles = d.pop("removeCurrentFuntionalRoles")

        removed_roles = cast(List[str], d.pop("removedRoles"))

        username = d.pop("username")

        campaign_roles_change = cls(
            added_roles=added_roles,
            remove_current_funtional_roles=remove_current_funtional_roles,
            removed_roles=removed_roles,
            username=username,
        )

        return campaign_roles_change



from http import HTTPStatus
from io import BytesIO
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.input_document import InputDocument
from ...types import UNSET, File, Response, Unset


def _get_kwargs(
    project_name: str,
    plan_name: str,
    *,
    client: Client,
    json_body: List["InputDocument"],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    debug: Union[Unset, None, bool] = False,
    parallelize: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/plans/{planName}/_annotate_format_documents".format(
        client.base_url, projectName=project_name, planName=plan_name
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["inlineLabels"] = inline_labels

    params["inlineLabelIds"] = inline_label_ids

    params["inlineText"] = inline_text

    params["debug"] = debug

    params["parallelize"] = parallelize

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = []
    for componentsschemas_input_document_array_item_data in json_body:
        componentsschemas_input_document_array_item = componentsschemas_input_document_array_item_data.to_dict()

        json_json_body.append(componentsschemas_input_document_array_item)

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[File]:
    if response.status_code == HTTPStatus.OK:
        response_200 = File(payload=BytesIO(response.json()))

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[File]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_name: str,
    plan_name: str,
    *,
    client: Client,
    json_body: List["InputDocument"],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    debug: Union[Unset, None, bool] = False,
    parallelize: Union[Unset, None, bool] = False,
) -> Response[File]:
    """annotate documents with multiple annotators and return formatted results in a zip

    Args:
        project_name (str):
        plan_name (str):
        inline_labels (Union[Unset, None, bool]):  Default: True.
        inline_label_ids (Union[Unset, None, bool]):  Default: True.
        inline_text (Union[Unset, None, bool]):  Default: True.
        debug (Union[Unset, None, bool]):
        parallelize (Union[Unset, None, bool]):
        json_body (List['InputDocument']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        plan_name=plan_name,
        client=client,
        json_body=json_body,
        inline_labels=inline_labels,
        inline_label_ids=inline_label_ids,
        inline_text=inline_text,
        debug=debug,
        parallelize=parallelize,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_name: str,
    plan_name: str,
    *,
    client: Client,
    json_body: List["InputDocument"],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    debug: Union[Unset, None, bool] = False,
    parallelize: Union[Unset, None, bool] = False,
) -> Optional[File]:
    """annotate documents with multiple annotators and return formatted results in a zip

    Args:
        project_name (str):
        plan_name (str):
        inline_labels (Union[Unset, None, bool]):  Default: True.
        inline_label_ids (Union[Unset, None, bool]):  Default: True.
        inline_text (Union[Unset, None, bool]):  Default: True.
        debug (Union[Unset, None, bool]):
        parallelize (Union[Unset, None, bool]):
        json_body (List['InputDocument']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File]
    """

    return sync_detailed(
        project_name=project_name,
        plan_name=plan_name,
        client=client,
        json_body=json_body,
        inline_labels=inline_labels,
        inline_label_ids=inline_label_ids,
        inline_text=inline_text,
        debug=debug,
        parallelize=parallelize,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    plan_name: str,
    *,
    client: Client,
    json_body: List["InputDocument"],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    debug: Union[Unset, None, bool] = False,
    parallelize: Union[Unset, None, bool] = False,
) -> Response[File]:
    """annotate documents with multiple annotators and return formatted results in a zip

    Args:
        project_name (str):
        plan_name (str):
        inline_labels (Union[Unset, None, bool]):  Default: True.
        inline_label_ids (Union[Unset, None, bool]):  Default: True.
        inline_text (Union[Unset, None, bool]):  Default: True.
        debug (Union[Unset, None, bool]):
        parallelize (Union[Unset, None, bool]):
        json_body (List['InputDocument']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        plan_name=plan_name,
        client=client,
        json_body=json_body,
        inline_labels=inline_labels,
        inline_label_ids=inline_label_ids,
        inline_text=inline_text,
        debug=debug,
        parallelize=parallelize,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_name: str,
    plan_name: str,
    *,
    client: Client,
    json_body: List["InputDocument"],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    debug: Union[Unset, None, bool] = False,
    parallelize: Union[Unset, None, bool] = False,
) -> Optional[File]:
    """annotate documents with multiple annotators and return formatted results in a zip

    Args:
        project_name (str):
        plan_name (str):
        inline_labels (Union[Unset, None, bool]):  Default: True.
        inline_label_ids (Union[Unset, None, bool]):  Default: True.
        inline_text (Union[Unset, None, bool]):  Default: True.
        debug (Union[Unset, None, bool]):
        parallelize (Union[Unset, None, bool]):
        json_body (List['InputDocument']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File]
    """

    return (
        await asyncio_detailed(
            project_name=project_name,
            plan_name=plan_name,
            client=client,
            json_body=json_body,
            inline_labels=inline_labels,
            inline_label_ids=inline_label_ids,
            inline_text=inline_text,
            debug=debug,
            parallelize=parallelize,
        )
    ).parsed



from enum import Enum


class SherpaJobBeanStatus(str, Enum):
    STARTED = "STARTED"
    PAUSED = "PAUSED"
    CANCELLED = "CANCELLED"
    FAILED = "FAILED"
    COMPLETED = "COMPLETED"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="DependsOn")


@attr.s(auto_attribs=True)
class DependsOn:
    """
    Attributes:
        auto (bool):
        groups (List[str]):
        hint (str):
        steps (List[str]):
    """

    auto: bool
    groups: List[str]
    hint: str
    steps: List[str]

    def to_dict(self) -> Dict[str, Any]:
        auto = self.auto
        groups = self.groups

        hint = self.hint
        steps = self.steps

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "auto": auto,
                "groups": groups,
                "hint": hint,
                "steps": steps,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        auto = d.pop("auto")

        groups = cast(List[str], d.pop("groups"))

        hint = d.pop("hint")

        steps = cast(List[str], d.pop("steps"))

        depends_on = cls(
            auto=auto,
            groups=groups,
            hint=hint,
            steps=steps,
        )

        return depends_on



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="LocalizedMessage")


@attr.s(auto_attribs=True)
class LocalizedMessage:
    """
    Attributes:
        body (str):
        title (str):
    """

    body: str
    title: str

    def to_dict(self) -> Dict[str, Any]:
        body = self.body
        title = self.title

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "body": body,
                "title": title,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        body = d.pop("body")

        title = d.pop("title")

        localized_message = cls(
            body=body,
            title=title,
        )

        return localized_message



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.localized_message import LocalizedMessage


T = TypeVar("T", bound="NewMessageLocalized")


@attr.s(auto_attribs=True)
class NewMessageLocalized:
    """ """

    additional_properties: Dict[str, "LocalizedMessage"] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        pass

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.localized_message import LocalizedMessage

        d = src_dict.copy()
        new_message_localized = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = LocalizedMessage.from_dict(prop_dict)

            additional_properties[prop_name] = additional_property

        new_message_localized.additional_properties = additional_properties
        return new_message_localized

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> "LocalizedMessage":
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: "LocalizedMessage") -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="CategoryAction")


@attr.s(auto_attribs=True)
class CategoryAction:
    """
    Attributes:
        add (bool): add or remove
        label_name (str): category label name
    """

    add: bool
    label_name: str

    def to_dict(self) -> Dict[str, Any]:
        add = self.add
        label_name = self.label_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "add": add,
                "labelName": label_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        add = d.pop("add")

        label_name = d.pop("labelName")

        category_action = cls(
            add=add,
            label_name=label_name,
        )

        return category_action



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.campaign_session import CampaignSession
    from ..models.campaign_session_mode import CampaignSessionMode
    from ..models.campaign_user_group import CampaignUserGroup
    from ..models.campaign_user_session import CampaignUserSession
    from ..models.email_notifications import EmailNotifications
    from ..models.message import Message
    from ..models.message_id import MessageId
    from ..models.new_message import NewMessage


T = TypeVar("T", bound="Campaign")


@attr.s(auto_attribs=True)
class Campaign:
    """
    Attributes:
        id (str):
        label (str):
        overview_messages (List[Union['Message', 'MessageId', 'NewMessage']]):
        stop_messages (List[Union['Message', 'MessageId', 'NewMessage']]):
        welcome_messages (List[Union['Message', 'MessageId', 'NewMessage']]):
        email_notifications (Union[Unset, EmailNotifications]):
        session_modes (Union[Unset, List['CampaignSessionMode']]):
        sessions (Union[Unset, List['CampaignSession']]):
        user_groups (Union[Unset, List['CampaignUserGroup']]):
        user_sessions (Union[Unset, List['CampaignUserSession']]):
    """

    id: str
    label: str
    overview_messages: List[Union["Message", "MessageId", "NewMessage"]]
    stop_messages: List[Union["Message", "MessageId", "NewMessage"]]
    welcome_messages: List[Union["Message", "MessageId", "NewMessage"]]
    email_notifications: Union[Unset, "EmailNotifications"] = UNSET
    session_modes: Union[Unset, List["CampaignSessionMode"]] = UNSET
    sessions: Union[Unset, List["CampaignSession"]] = UNSET
    user_groups: Union[Unset, List["CampaignUserGroup"]] = UNSET
    user_sessions: Union[Unset, List["CampaignUserSession"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        from ..models.message import Message
        from ..models.message_id import MessageId

        id = self.id
        label = self.label
        overview_messages = []
        for overview_messages_item_data in self.overview_messages:
            overview_messages_item: Dict[str, Any]

            if isinstance(overview_messages_item_data, MessageId):
                overview_messages_item = overview_messages_item_data.to_dict()

            elif isinstance(overview_messages_item_data, Message):
                overview_messages_item = overview_messages_item_data.to_dict()

            else:
                overview_messages_item = overview_messages_item_data.to_dict()

            overview_messages.append(overview_messages_item)

        stop_messages = []
        for stop_messages_item_data in self.stop_messages:
            stop_messages_item: Dict[str, Any]

            if isinstance(stop_messages_item_data, MessageId):
                stop_messages_item = stop_messages_item_data.to_dict()

            elif isinstance(stop_messages_item_data, Message):
                stop_messages_item = stop_messages_item_data.to_dict()

            else:
                stop_messages_item = stop_messages_item_data.to_dict()

            stop_messages.append(stop_messages_item)

        welcome_messages = []
        for welcome_messages_item_data in self.welcome_messages:
            welcome_messages_item: Dict[str, Any]

            if isinstance(welcome_messages_item_data, MessageId):
                welcome_messages_item = welcome_messages_item_data.to_dict()

            elif isinstance(welcome_messages_item_data, Message):
                welcome_messages_item = welcome_messages_item_data.to_dict()

            else:
                welcome_messages_item = welcome_messages_item_data.to_dict()

            welcome_messages.append(welcome_messages_item)

        email_notifications: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.email_notifications, Unset):
            email_notifications = self.email_notifications.to_dict()

        session_modes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.session_modes, Unset):
            session_modes = []
            for session_modes_item_data in self.session_modes:
                session_modes_item = session_modes_item_data.to_dict()

                session_modes.append(session_modes_item)

        sessions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sessions, Unset):
            sessions = []
            for sessions_item_data in self.sessions:
                sessions_item = sessions_item_data.to_dict()

                sessions.append(sessions_item)

        user_groups: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.user_groups, Unset):
            user_groups = []
            for user_groups_item_data in self.user_groups:
                user_groups_item = user_groups_item_data.to_dict()

                user_groups.append(user_groups_item)

        user_sessions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.user_sessions, Unset):
            user_sessions = []
            for user_sessions_item_data in self.user_sessions:
                user_sessions_item = user_sessions_item_data.to_dict()

                user_sessions.append(user_sessions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "label": label,
                "overviewMessages": overview_messages,
                "stopMessages": stop_messages,
                "welcomeMessages": welcome_messages,
            }
        )
        if email_notifications is not UNSET:
            field_dict["emailNotifications"] = email_notifications
        if session_modes is not UNSET:
            field_dict["sessionModes"] = session_modes
        if sessions is not UNSET:
            field_dict["sessions"] = sessions
        if user_groups is not UNSET:
            field_dict["userGroups"] = user_groups
        if user_sessions is not UNSET:
            field_dict["userSessions"] = user_sessions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.campaign_session import CampaignSession
        from ..models.campaign_session_mode import CampaignSessionMode
        from ..models.campaign_user_group import CampaignUserGroup
        from ..models.campaign_user_session import CampaignUserSession
        from ..models.email_notifications import EmailNotifications
        from ..models.message import Message
        from ..models.message_id import MessageId
        from ..models.new_message import NewMessage

        d = src_dict.copy()
        id = d.pop("id")

        label = d.pop("label")

        overview_messages = []
        _overview_messages = d.pop("overviewMessages")
        for overview_messages_item_data in _overview_messages:

            def _parse_overview_messages_item(data: object) -> Union["Message", "MessageId", "NewMessage"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    overview_messages_item_type_0 = MessageId.from_dict(data)

                    return overview_messages_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    overview_messages_item_type_1 = Message.from_dict(data)

                    return overview_messages_item_type_1
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                overview_messages_item_type_2 = NewMessage.from_dict(data)

                return overview_messages_item_type_2

            overview_messages_item = _parse_overview_messages_item(overview_messages_item_data)

            overview_messages.append(overview_messages_item)

        stop_messages = []
        _stop_messages = d.pop("stopMessages")
        for stop_messages_item_data in _stop_messages:

            def _parse_stop_messages_item(data: object) -> Union["Message", "MessageId", "NewMessage"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    stop_messages_item_type_0 = MessageId.from_dict(data)

                    return stop_messages_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    stop_messages_item_type_1 = Message.from_dict(data)

                    return stop_messages_item_type_1
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                stop_messages_item_type_2 = NewMessage.from_dict(data)

                return stop_messages_item_type_2

            stop_messages_item = _parse_stop_messages_item(stop_messages_item_data)

            stop_messages.append(stop_messages_item)

        welcome_messages = []
        _welcome_messages = d.pop("welcomeMessages")
        for welcome_messages_item_data in _welcome_messages:

            def _parse_welcome_messages_item(data: object) -> Union["Message", "MessageId", "NewMessage"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    welcome_messages_item_type_0 = MessageId.from_dict(data)

                    return welcome_messages_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    welcome_messages_item_type_1 = Message.from_dict(data)

                    return welcome_messages_item_type_1
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                welcome_messages_item_type_2 = NewMessage.from_dict(data)

                return welcome_messages_item_type_2

            welcome_messages_item = _parse_welcome_messages_item(welcome_messages_item_data)

            welcome_messages.append(welcome_messages_item)

        _email_notifications = d.pop("emailNotifications", UNSET)
        email_notifications: Union[Unset, EmailNotifications]
        if isinstance(_email_notifications, Unset):
            email_notifications = UNSET
        else:
            email_notifications = EmailNotifications.from_dict(_email_notifications)

        session_modes = []
        _session_modes = d.pop("sessionModes", UNSET)
        for session_modes_item_data in _session_modes or []:
            session_modes_item = CampaignSessionMode.from_dict(session_modes_item_data)

            session_modes.append(session_modes_item)

        sessions = []
        _sessions = d.pop("sessions", UNSET)
        for sessions_item_data in _sessions or []:
            sessions_item = CampaignSession.from_dict(sessions_item_data)

            sessions.append(sessions_item)

        user_groups = []
        _user_groups = d.pop("userGroups", UNSET)
        for user_groups_item_data in _user_groups or []:
            user_groups_item = CampaignUserGroup.from_dict(user_groups_item_data)

            user_groups.append(user_groups_item)

        user_sessions = []
        _user_sessions = d.pop("userSessions", UNSET)
        for user_sessions_item_data in _user_sessions or []:
            user_sessions_item = CampaignUserSession.from_dict(user_sessions_item_data)

            user_sessions.append(user_sessions_item)

        campaign = cls(
            id=id,
            label=label,
            overview_messages=overview_messages,
            stop_messages=stop_messages,
            welcome_messages=welcome_messages,
            email_notifications=email_notifications,
            session_modes=session_modes,
            sessions=sessions,
            user_groups=user_groups,
            user_sessions=user_sessions,
        )

        return campaign



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.annotation_facets import AnnotationFacets
    from ..models.document_facets import DocumentFacets
    from ..models.suggestion_facets import SuggestionFacets


T = TypeVar("T", bound="AnnotationMetrics")


@attr.s(auto_attribs=True)
class AnnotationMetrics:
    """
    Attributes:
        annotation_count (int):
        annotation_facets (AnnotationFacets):
        document_facets (DocumentFacets):
        documents_in_dataset (int):
        segments_in_dataset (int):
        suggestion_count (int):
        suggestion_facets (SuggestionFacets):
    """

    annotation_count: int
    annotation_facets: "AnnotationFacets"
    document_facets: "DocumentFacets"
    documents_in_dataset: int
    segments_in_dataset: int
    suggestion_count: int
    suggestion_facets: "SuggestionFacets"

    def to_dict(self) -> Dict[str, Any]:
        annotation_count = self.annotation_count
        annotation_facets = self.annotation_facets.to_dict()

        document_facets = self.document_facets.to_dict()

        documents_in_dataset = self.documents_in_dataset
        segments_in_dataset = self.segments_in_dataset
        suggestion_count = self.suggestion_count
        suggestion_facets = self.suggestion_facets.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "annotationCount": annotation_count,
                "annotationFacets": annotation_facets,
                "documentFacets": document_facets,
                "documentsInDataset": documents_in_dataset,
                "segmentsInDataset": segments_in_dataset,
                "suggestionCount": suggestion_count,
                "suggestionFacets": suggestion_facets,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotation_facets import AnnotationFacets
        from ..models.document_facets import DocumentFacets
        from ..models.suggestion_facets import SuggestionFacets

        d = src_dict.copy()
        annotation_count = d.pop("annotationCount")

        annotation_facets = AnnotationFacets.from_dict(d.pop("annotationFacets"))

        document_facets = DocumentFacets.from_dict(d.pop("documentFacets"))

        documents_in_dataset = d.pop("documentsInDataset")

        segments_in_dataset = d.pop("segmentsInDataset")

        suggestion_count = d.pop("suggestionCount")

        suggestion_facets = SuggestionFacets.from_dict(d.pop("suggestionFacets"))

        annotation_metrics = cls(
            annotation_count=annotation_count,
            annotation_facets=annotation_facets,
            document_facets=document_facets,
            documents_in_dataset=documents_in_dataset,
            segments_in_dataset=segments_in_dataset,
            suggestion_count=suggestion_count,
            suggestion_facets=suggestion_facets,
        )

        return annotation_metrics



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.annotated_doc_annotation_creation_mode import AnnotatedDocAnnotationCreationMode
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.annotated_doc_annotation_properties import AnnotatedDocAnnotationProperties
    from ..models.annotation_term import AnnotationTerm


T = TypeVar("T", bound="AnnotatedDocAnnotation")


@attr.s(auto_attribs=True)
class AnnotatedDocAnnotation:
    """A document annotation

    Attributes:
        end (int): End offset in document
        label_name (str): Label name
        start (int): Start offset in document
        text (str): Covered text
        creation_mode (Union[Unset, AnnotatedDocAnnotationCreationMode]): Creation mode
        label (Union[Unset, str]): Human-friendly label
        label_id (Union[Unset, str]): External label identifier
        properties (Union[Unset, AnnotatedDocAnnotationProperties]): Additional properties
        score (Union[Unset, float]): Score of the annotation
        terms (Union[Unset, List['AnnotationTerm']]):
    """

    end: int
    label_name: str
    start: int
    text: str
    creation_mode: Union[Unset, AnnotatedDocAnnotationCreationMode] = UNSET
    label: Union[Unset, str] = UNSET
    label_id: Union[Unset, str] = UNSET
    properties: Union[Unset, "AnnotatedDocAnnotationProperties"] = UNSET
    score: Union[Unset, float] = UNSET
    terms: Union[Unset, List["AnnotationTerm"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        end = self.end
        label_name = self.label_name
        start = self.start
        text = self.text
        creation_mode: Union[Unset, str] = UNSET
        if not isinstance(self.creation_mode, Unset):
            creation_mode = self.creation_mode.value

        label = self.label
        label_id = self.label_id
        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        score = self.score
        terms: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.terms, Unset):
            terms = []
            for terms_item_data in self.terms:
                terms_item = terms_item_data.to_dict()

                terms.append(terms_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "end": end,
                "labelName": label_name,
                "start": start,
                "text": text,
            }
        )
        if creation_mode is not UNSET:
            field_dict["creationMode"] = creation_mode
        if label is not UNSET:
            field_dict["label"] = label
        if label_id is not UNSET:
            field_dict["labelId"] = label_id
        if properties is not UNSET:
            field_dict["properties"] = properties
        if score is not UNSET:
            field_dict["score"] = score
        if terms is not UNSET:
            field_dict["terms"] = terms

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotated_doc_annotation_properties import AnnotatedDocAnnotationProperties
        from ..models.annotation_term import AnnotationTerm

        d = src_dict.copy()
        end = d.pop("end")

        label_name = d.pop("labelName")

        start = d.pop("start")

        text = d.pop("text")

        _creation_mode = d.pop("creationMode", UNSET)
        creation_mode: Union[Unset, AnnotatedDocAnnotationCreationMode]
        if isinstance(_creation_mode, Unset):
            creation_mode = UNSET
        else:
            creation_mode = AnnotatedDocAnnotationCreationMode(_creation_mode)

        label = d.pop("label", UNSET)

        label_id = d.pop("labelId", UNSET)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, AnnotatedDocAnnotationProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = AnnotatedDocAnnotationProperties.from_dict(_properties)

        score = d.pop("score", UNSET)

        terms = []
        _terms = d.pop("terms", UNSET)
        for terms_item_data in _terms or []:
            terms_item = AnnotationTerm.from_dict(terms_item_data)

            terms.append(terms_item)

        annotated_doc_annotation = cls(
            end=end,
            label_name=label_name,
            start=start,
            text=text,
            creation_mode=creation_mode,
            label=label,
            label_id=label_id,
            properties=properties,
            score=score,
            terms=terms,
        )

        return annotated_doc_annotation



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.message_localized import MessageLocalized
    from ..models.message_mark import MessageMark


T = TypeVar("T", bound="Message")


@attr.s(auto_attribs=True)
class Message:
    """
    Attributes:
        id (str):
        localized (MessageLocalized):
        group (Union[Unset, str]):
        index (Union[Unset, int]):
        mark (Union[Unset, MessageMark]):
    """

    id: str
    localized: "MessageLocalized"
    group: Union[Unset, str] = UNSET
    index: Union[Unset, int] = UNSET
    mark: Union[Unset, "MessageMark"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        localized = self.localized.to_dict()

        group = self.group
        index = self.index
        mark: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mark, Unset):
            mark = self.mark.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "localized": localized,
            }
        )
        if group is not UNSET:
            field_dict["group"] = group
        if index is not UNSET:
            field_dict["index"] = index
        if mark is not UNSET:
            field_dict["mark"] = mark

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message_localized import MessageLocalized
        from ..models.message_mark import MessageMark

        d = src_dict.copy()
        id = d.pop("id")

        localized = MessageLocalized.from_dict(d.pop("localized"))

        group = d.pop("group", UNSET)

        index = d.pop("index", UNSET)

        _mark = d.pop("mark", UNSET)
        mark: Union[Unset, MessageMark]
        if isinstance(_mark, Unset):
            mark = UNSET
        else:
            mark = MessageMark.from_dict(_mark)

        message = cls(
            id=id,
            localized=localized,
            group=group,
            index=index,
            mark=mark,
        )

        return message



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmailNotifications")


@attr.s(auto_attribs=True)
class EmailNotifications:
    """
    Attributes:
        enabled (bool):
        notified_users (Union[Unset, List[str]]):
    """

    enabled: bool
    notified_users: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        notified_users: Union[Unset, List[str]] = UNSET
        if not isinstance(self.notified_users, Unset):
            notified_users = self.notified_users

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "enabled": enabled,
            }
        )
        if notified_users is not UNSET:
            field_dict["notifiedUsers"] = notified_users

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled")

        notified_users = cast(List[str], d.pop("notifiedUsers", UNSET))

        email_notifications = cls(
            enabled=enabled,
            notified_users=notified_users,
        )

        return email_notifications



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserProfile")


@attr.s(auto_attribs=True)
class UserProfile:
    """
    Attributes:
        profilename (str):
        username (str):
        email (Union[Unset, str]):
    """

    profilename: str
    username: str
    email: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        profilename = self.profilename
        username = self.username
        email = self.email

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "profilename": profilename,
                "username": username,
            }
        )
        if email is not UNSET:
            field_dict["email"] = email

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        profilename = d.pop("profilename")

        username = d.pop("username")

        email = d.pop("email", UNSET)

        user_profile = cls(
            profilename=profilename,
            username=username,
            email=email,
        )

        return user_profile



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="EngineConfig")


@attr.s(auto_attribs=True)
class EngineConfig:
    """
    Attributes:
        name (str):
        type (str):
    """

    name: str
    type: str

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        type = d.pop("type")

        engine_config = cls(
            name=name,
            type=type,
        )

        return engine_config



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.bucket import Bucket


T = TypeVar("T", bound="Aggregation")


@attr.s(auto_attribs=True)
class Aggregation:
    """
    Attributes:
        buckets (List['Bucket']):
        name (str):
    """

    buckets: List["Bucket"]
    name: str

    def to_dict(self) -> Dict[str, Any]:
        buckets = []
        for buckets_item_data in self.buckets:
            buckets_item = buckets_item_data.to_dict()

            buckets.append(buckets_item)

        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "buckets": buckets,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.bucket import Bucket

        d = src_dict.copy()
        buckets = []
        _buckets = d.pop("buckets")
        for buckets_item_data in _buckets:
            buckets_item = Bucket.from_dict(buckets_item_data)

            buckets.append(buckets_item)

        name = d.pop("name")

        aggregation = cls(
            buckets=buckets,
            name=name,
        )

        return aggregation



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.project_config_creation_properties import ProjectConfigCreationProperties


T = TypeVar("T", bound="ProjectConfigCreation")


@attr.s(auto_attribs=True)
class ProjectConfigCreation:
    """
    Attributes:
        collaborative_annotation (Union[Unset, bool]):
        description (Union[Unset, str]):
        image_filename (Union[Unset, str]):
        image_id (Union[Unset, str]):
        image_url (Union[Unset, str]):
        label (Union[Unset, str]):
        lang (Union[Unset, str]):  Default: 'en'.
        metafacets (Union[Unset, str]):
        name (Union[Unset, str]):
        nature (Union[Unset, str]):  Default: 'sequence_labelling'.
        properties (Union[Unset, ProjectConfigCreationProperties]):
    """

    collaborative_annotation: Union[Unset, bool] = UNSET
    description: Union[Unset, str] = UNSET
    image_filename: Union[Unset, str] = UNSET
    image_id: Union[Unset, str] = UNSET
    image_url: Union[Unset, str] = UNSET
    label: Union[Unset, str] = UNSET
    lang: Union[Unset, str] = "en"
    metafacets: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    nature: Union[Unset, str] = "sequence_labelling"
    properties: Union[Unset, "ProjectConfigCreationProperties"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        collaborative_annotation = self.collaborative_annotation
        description = self.description
        image_filename = self.image_filename
        image_id = self.image_id
        image_url = self.image_url
        label = self.label
        lang = self.lang
        metafacets = self.metafacets
        name = self.name
        nature = self.nature
        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if collaborative_annotation is not UNSET:
            field_dict["collaborativeAnnotation"] = collaborative_annotation
        if description is not UNSET:
            field_dict["description"] = description
        if image_filename is not UNSET:
            field_dict["imageFilename"] = image_filename
        if image_id is not UNSET:
            field_dict["imageId"] = image_id
        if image_url is not UNSET:
            field_dict["imageUrl"] = image_url
        if label is not UNSET:
            field_dict["label"] = label
        if lang is not UNSET:
            field_dict["lang"] = lang
        if metafacets is not UNSET:
            field_dict["metafacets"] = metafacets
        if name is not UNSET:
            field_dict["name"] = name
        if nature is not UNSET:
            field_dict["nature"] = nature
        if properties is not UNSET:
            field_dict["properties"] = properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.project_config_creation_properties import ProjectConfigCreationProperties

        d = src_dict.copy()
        collaborative_annotation = d.pop("collaborativeAnnotation", UNSET)

        description = d.pop("description", UNSET)

        image_filename = d.pop("imageFilename", UNSET)

        image_id = d.pop("imageId", UNSET)

        image_url = d.pop("imageUrl", UNSET)

        label = d.pop("label", UNSET)

        lang = d.pop("lang", UNSET)

        metafacets = d.pop("metafacets", UNSET)

        name = d.pop("name", UNSET)

        nature = d.pop("nature", UNSET)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, ProjectConfigCreationProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = ProjectConfigCreationProperties.from_dict(_properties)

        project_config_creation = cls(
            collaborative_annotation=collaborative_annotation,
            description=description,
            image_filename=image_filename,
            image_id=image_id,
            image_url=image_url,
            label=label,
            lang=lang,
            metafacets=metafacets,
            name=name,
            nature=nature,
            properties=properties,
        )

        return project_config_creation



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.share_mode import ShareMode


T = TypeVar("T", bound="GroupShare")


@attr.s(auto_attribs=True)
class GroupShare:
    """
    Attributes:
        can_revoke (bool):
        group_name (str):
        mode (ShareMode):
    """

    can_revoke: bool
    group_name: str
    mode: "ShareMode"

    def to_dict(self) -> Dict[str, Any]:
        can_revoke = self.can_revoke
        group_name = self.group_name
        mode = self.mode.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "canRevoke": can_revoke,
                "groupName": group_name,
                "mode": mode,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.share_mode import ShareMode

        d = src_dict.copy()
        can_revoke = d.pop("canRevoke")

        group_name = d.pop("groupName")

        mode = ShareMode.from_dict(d.pop("mode"))

        group_share = cls(
            can_revoke=can_revoke,
            group_name=group_name,
            mode=mode,
        )

        return group_share



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectProperty")


@attr.s(auto_attribs=True)
class ProjectProperty:
    """
    Attributes:
        name (Union[Unset, str]):  Default: 'value'.
    """

    name: Union[Unset, str] = "value"

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        project_property = cls(
            name=name,
        )

        return project_property



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="CategoryId")


@attr.s(auto_attribs=True)
class CategoryId:
    """
    Attributes:
        identifier (str):
    """

    identifier: str

    def to_dict(self) -> Dict[str, Any]:
        identifier = self.identifier

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "identifier": identifier,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        identifier = d.pop("identifier")

        category_id = cls(
            identifier=identifier,
        )

        return category_id



from enum import Enum


class UserSessionActionAction(str, Enum):
    CREATE = "create"
    OPEN = "open"
    START = "start"
    PAUSE = "pause"
    STOP = "stop"
    CLOSE = "close"
    ADD_EXTRA_TIME = "add_extra_time"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.annotation_plan import AnnotationPlan


T = TypeVar("T", bound="NamedAnnotationPlan")


@attr.s(auto_attribs=True)
class NamedAnnotationPlan:
    """
    Attributes:
        label (str):
        name (str):
        parameters (AnnotationPlan):
        created_at (Union[Unset, str]):
        created_by (Union[Unset, str]):
        favorite (Union[Unset, bool]):
        modified_at (Union[Unset, str]):
        modified_by (Union[Unset, str]):
        tags (Union[Unset, List[str]]):
    """

    label: str
    name: str
    parameters: "AnnotationPlan"
    created_at: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    favorite: Union[Unset, bool] = UNSET
    modified_at: Union[Unset, str] = UNSET
    modified_by: Union[Unset, str] = UNSET
    tags: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        name = self.name
        parameters = self.parameters.to_dict()

        created_at = self.created_at
        created_by = self.created_by
        favorite = self.favorite
        modified_at = self.modified_at
        modified_by = self.modified_by
        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "label": label,
                "name": name,
                "parameters": parameters,
            }
        )
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if favorite is not UNSET:
            field_dict["favorite"] = favorite
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if modified_by is not UNSET:
            field_dict["modifiedBy"] = modified_by
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotation_plan import AnnotationPlan

        d = src_dict.copy()
        label = d.pop("label")

        name = d.pop("name")

        parameters = AnnotationPlan.from_dict(d.pop("parameters"))

        created_at = d.pop("createdAt", UNSET)

        created_by = d.pop("createdBy", UNSET)

        favorite = d.pop("favorite", UNSET)

        modified_at = d.pop("modifiedAt", UNSET)

        modified_by = d.pop("modifiedBy", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        named_annotation_plan = cls(
            label=label,
            name=name,
            parameters=parameters,
            created_at=created_at,
            created_by=created_by,
            favorite=favorite,
            modified_at=modified_at,
            modified_by=modified_by,
            tags=tags,
        )

        return named_annotation_plan



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.document import Document
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    query_filter: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    output_fields: Union[Unset, None, str] = "",
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/documents/_search_and_export".format(client.base_url, projectName=project_name)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["query"] = query

    params["queryFilter"] = query_filter

    params["simpleQuery"] = simple_query

    params["outputFields"] = output_fields

    json_selected_facets: Union[Unset, None, List[str]] = UNSET
    if not isinstance(selected_facets, Unset):
        if selected_facets is None:
            json_selected_facets = None
        else:
            json_selected_facets = selected_facets

    params["selectedFacets"] = json_selected_facets

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[List["Document"]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for componentsschemas_document_array_item_data in _response_200:
            componentsschemas_document_array_item = Document.from_dict(componentsschemas_document_array_item_data)

            response_200.append(componentsschemas_document_array_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[List["Document"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    query_filter: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    output_fields: Union[Unset, None, str] = "",
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Response[List["Document"]]:
    """Search for documents and export them

    Args:
        project_name (str):
        query (Union[Unset, None, str]):  Default: ''.
        query_filter (Union[Unset, None, str]):  Default: ''.
        simple_query (Union[Unset, None, bool]):
        output_fields (Union[Unset, None, str]):  Default: ''.
        selected_facets (Union[Unset, None, List[str]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Document']]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        query=query,
        query_filter=query_filter,
        simple_query=simple_query,
        output_fields=output_fields,
        selected_facets=selected_facets,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    query_filter: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    output_fields: Union[Unset, None, str] = "",
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Optional[List["Document"]]:
    """Search for documents and export them

    Args:
        project_name (str):
        query (Union[Unset, None, str]):  Default: ''.
        query_filter (Union[Unset, None, str]):  Default: ''.
        simple_query (Union[Unset, None, bool]):
        output_fields (Union[Unset, None, str]):  Default: ''.
        selected_facets (Union[Unset, None, List[str]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Document']]
    """

    return sync_detailed(
        project_name=project_name,
        client=client,
        query=query,
        query_filter=query_filter,
        simple_query=simple_query,
        output_fields=output_fields,
        selected_facets=selected_facets,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    query_filter: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    output_fields: Union[Unset, None, str] = "",
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Response[List["Document"]]:
    """Search for documents and export them

    Args:
        project_name (str):
        query (Union[Unset, None, str]):  Default: ''.
        query_filter (Union[Unset, None, str]):  Default: ''.
        simple_query (Union[Unset, None, bool]):
        output_fields (Union[Unset, None, str]):  Default: ''.
        selected_facets (Union[Unset, None, List[str]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Document']]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        query=query,
        query_filter=query_filter,
        simple_query=simple_query,
        output_fields=output_fields,
        selected_facets=selected_facets,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    query_filter: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    output_fields: Union[Unset, None, str] = "",
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Optional[List["Document"]]:
    """Search for documents and export them

    Args:
        project_name (str):
        query (Union[Unset, None, str]):  Default: ''.
        query_filter (Union[Unset, None, str]):  Default: ''.
        simple_query (Union[Unset, None, bool]):
        output_fields (Union[Unset, None, str]):  Default: ''.
        selected_facets (Union[Unset, None, List[str]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Document']]
    """

    return (
        await asyncio_detailed(
            project_name=project_name,
            client=client,
            query=query,
            query_filter=query_filter,
            simple_query=simple_query,
            output_fields=output_fields,
            selected_facets=selected_facets,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.gazetteer_parameters import GazetteerParameters


T = TypeVar("T", bound="Gazetteer")


@attr.s(auto_attribs=True)
class Gazetteer:
    """
    Attributes:
        duration (int):
        engine (str):
        label (str):
        models (int):
        name (str):
        parameters (GazetteerParameters):
        running (bool):
        timestamp (int):
        uptodate (bool):
        email_notification (Union[Unset, bool]):
        favorite (Union[Unset, bool]):
        tags (Union[Unset, List[str]]):
    """

    duration: int
    engine: str
    label: str
    models: int
    name: str
    parameters: "GazetteerParameters"
    running: bool
    timestamp: int
    uptodate: bool
    email_notification: Union[Unset, bool] = UNSET
    favorite: Union[Unset, bool] = UNSET
    tags: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        duration = self.duration
        engine = self.engine
        label = self.label
        models = self.models
        name = self.name
        parameters = self.parameters.to_dict()

        running = self.running
        timestamp = self.timestamp
        uptodate = self.uptodate
        email_notification = self.email_notification
        favorite = self.favorite
        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "duration": duration,
                "engine": engine,
                "label": label,
                "models": models,
                "name": name,
                "parameters": parameters,
                "running": running,
                "timestamp": timestamp,
                "uptodate": uptodate,
            }
        )
        if email_notification is not UNSET:
            field_dict["emailNotification"] = email_notification
        if favorite is not UNSET:
            field_dict["favorite"] = favorite
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.gazetteer_parameters import GazetteerParameters

        d = src_dict.copy()
        duration = d.pop("duration")

        engine = d.pop("engine")

        label = d.pop("label")

        models = d.pop("models")

        name = d.pop("name")

        parameters = GazetteerParameters.from_dict(d.pop("parameters"))

        running = d.pop("running")

        timestamp = d.pop("timestamp")

        uptodate = d.pop("uptodate")

        email_notification = d.pop("emailNotification", UNSET)

        favorite = d.pop("favorite", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        gazetteer = cls(
            duration=duration,
            engine=engine,
            label=label,
            models=models,
            name=name,
            parameters=parameters,
            running=running,
            timestamp=timestamp,
            uptodate=uptodate,
            email_notification=email_notification,
            favorite=favorite,
            tags=tags,
        )

        return gazetteer



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserResponse")


@attr.s(auto_attribs=True)
class UserResponse:
    """
    Attributes:
        profilename (str):
        username (str):
        created_at (Union[Unset, str]):
        created_by (Union[Unset, str]):
        default_group (Union[Unset, str]):
        email (Union[Unset, str]):
        groups (Union[Unset, List[str]]):
        permissions (Union[Unset, List[str]]):
        roles (Union[Unset, List[str]]):
    """

    profilename: str
    username: str
    created_at: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    default_group: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    groups: Union[Unset, List[str]] = UNSET
    permissions: Union[Unset, List[str]] = UNSET
    roles: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        profilename = self.profilename
        username = self.username
        created_at = self.created_at
        created_by = self.created_by
        default_group = self.default_group
        email = self.email
        groups: Union[Unset, List[str]] = UNSET
        if not isinstance(self.groups, Unset):
            groups = self.groups

        permissions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = self.permissions

        roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.roles, Unset):
            roles = self.roles

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "profilename": profilename,
                "username": username,
            }
        )
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if default_group is not UNSET:
            field_dict["defaultGroup"] = default_group
        if email is not UNSET:
            field_dict["email"] = email
        if groups is not UNSET:
            field_dict["groups"] = groups
        if permissions is not UNSET:
            field_dict["permissions"] = permissions
        if roles is not UNSET:
            field_dict["roles"] = roles

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        profilename = d.pop("profilename")

        username = d.pop("username")

        created_at = d.pop("createdAt", UNSET)

        created_by = d.pop("createdBy", UNSET)

        default_group = d.pop("defaultGroup", UNSET)

        email = d.pop("email", UNSET)

        groups = cast(List[str], d.pop("groups", UNSET))

        permissions = cast(List[str], d.pop("permissions", UNSET))

        roles = cast(List[str], d.pop("roles", UNSET))

        user_response = cls(
            profilename=profilename,
            username=username,
            created_at=created_at,
            created_by=created_by,
            default_group=default_group,
            email=email,
            groups=groups,
            permissions=permissions,
            roles=roles,
        )

        return user_response



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import Client
from ...models.ack import Ack
from ...types import Response


def _get_kwargs(
    project_name: str,
    message_id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/messages/{messageId}".format(
        client.base_url, projectName=project_name, messageId=message_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "delete",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Union[Ack, Any]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Ack.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = cast(Any, None)
        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Union[Ack, Any]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_name: str,
    message_id: str,
    *,
    client: Client,
) -> Response[Union[Ack, Any]]:
    """delete message

    Args:
        project_name (str):
        message_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Ack, Any]]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        message_id=message_id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_name: str,
    message_id: str,
    *,
    client: Client,
) -> Optional[Union[Ack, Any]]:
    """delete message

    Args:
        project_name (str):
        message_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Ack, Any]]
    """

    return sync_detailed(
        project_name=project_name,
        message_id=message_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    message_id: str,
    *,
    client: Client,
) -> Response[Union[Ack, Any]]:
    """delete message

    Args:
        project_name (str):
        message_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Ack, Any]]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        message_id=message_id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_name: str,
    message_id: str,
    *,
    client: Client,
) -> Optional[Union[Ack, Any]]:
    """delete message

    Args:
        project_name (str):
        message_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Ack, Any]]
    """

    return (
        await asyncio_detailed(
            project_name=project_name,
            message_id=message_id,
            client=client,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.new_message_localized import NewMessageLocalized


T = TypeVar("T", bound="NewMessage")


@attr.s(auto_attribs=True)
class NewMessage:
    """
    Attributes:
        localized (NewMessageLocalized):
        group (Union[Unset, str]):
        index (Union[Unset, int]):
        scope (Union[Unset, str]):
    """

    localized: "NewMessageLocalized"
    group: Union[Unset, str] = UNSET
    index: Union[Unset, int] = UNSET
    scope: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        localized = self.localized.to_dict()

        group = self.group
        index = self.index
        scope = self.scope

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "localized": localized,
            }
        )
        if group is not UNSET:
            field_dict["group"] = group
        if index is not UNSET:
            field_dict["index"] = index
        if scope is not UNSET:
            field_dict["scope"] = scope

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.new_message_localized import NewMessageLocalized

        d = src_dict.copy()
        localized = NewMessageLocalized.from_dict(d.pop("localized"))

        group = d.pop("group", UNSET)

        index = d.pop("index", UNSET)

        scope = d.pop("scope", UNSET)

        new_message = cls(
            localized=localized,
            group=group,
            index=index,
            scope=scope,
        )

        return new_message



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.http_service_metadata import HttpServiceMetadata


T = TypeVar("T", bound="HttpServiceRecord")


@attr.s(auto_attribs=True)
class HttpServiceRecord:
    """
    Attributes:
        host (str):
        metadata (HttpServiceMetadata):
        name (str):
        port (int):
        ssl (Union[Unset, bool]):
    """

    host: str
    metadata: "HttpServiceMetadata"
    name: str
    port: int
    ssl: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        host = self.host
        metadata = self.metadata.to_dict()

        name = self.name
        port = self.port
        ssl = self.ssl

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "host": host,
                "metadata": metadata,
                "name": name,
                "port": port,
            }
        )
        if ssl is not UNSET:
            field_dict["ssl"] = ssl

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.http_service_metadata import HttpServiceMetadata

        d = src_dict.copy()
        host = d.pop("host")

        metadata = HttpServiceMetadata.from_dict(d.pop("metadata"))

        name = d.pop("name")

        port = d.pop("port")

        ssl = d.pop("ssl", UNSET)

        http_service_record = cls(
            host=host,
            metadata=metadata,
            name=name,
            port=port,
            ssl=ssl,
        )

        return http_service_record



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.aggregation import Aggregation
    from ..models.document_hit import DocumentHit
    from ..models.search_total import SearchTotal


T = TypeVar("T", bound="DocumentHits")


@attr.s(auto_attribs=True)
class DocumentHits:
    """
    Attributes:
        hits (List['DocumentHit']):
        total (SearchTotal):
        aggregations (Union[Unset, List['Aggregation']]):
        max_score (Union[Unset, float]):
    """

    hits: List["DocumentHit"]
    total: "SearchTotal"
    aggregations: Union[Unset, List["Aggregation"]] = UNSET
    max_score: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        hits = []
        for hits_item_data in self.hits:
            hits_item = hits_item_data.to_dict()

            hits.append(hits_item)

        total = self.total.to_dict()

        aggregations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.aggregations, Unset):
            aggregations = []
            for aggregations_item_data in self.aggregations:
                aggregations_item = aggregations_item_data.to_dict()

                aggregations.append(aggregations_item)

        max_score = self.max_score

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "hits": hits,
                "total": total,
            }
        )
        if aggregations is not UNSET:
            field_dict["aggregations"] = aggregations
        if max_score is not UNSET:
            field_dict["max_score"] = max_score

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.aggregation import Aggregation
        from ..models.document_hit import DocumentHit
        from ..models.search_total import SearchTotal

        d = src_dict.copy()
        hits = []
        _hits = d.pop("hits")
        for hits_item_data in _hits:
            hits_item = DocumentHit.from_dict(hits_item_data)

            hits.append(hits_item)

        total = SearchTotal.from_dict(d.pop("total"))

        aggregations = []
        _aggregations = d.pop("aggregations", UNSET)
        for aggregations_item_data in _aggregations or []:
            aggregations_item = Aggregation.from_dict(aggregations_item_data)

            aggregations.append(aggregations_item)

        max_score = d.pop("max_score", UNSET)

        document_hits = cls(
            hits=hits,
            total=total,
            aggregations=aggregations,
            max_score=max_score,
        )

        return document_hits



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.campaign_user_group import CampaignUserGroup
    from ..models.email_notifications import EmailNotifications
    from ..models.new_campaign_session import NewCampaignSession
    from ..models.new_campaign_session_mode import NewCampaignSessionMode
    from ..models.new_message import NewMessage


T = TypeVar("T", bound="NewCampaign")


@attr.s(auto_attribs=True)
class NewCampaign:
    """
    Attributes:
        label (str):
        email_notifications (Union[Unset, EmailNotifications]):
        overview_messages (Union[Unset, List['NewMessage']]):
        session_modes (Union[Unset, List['NewCampaignSessionMode']]):
        sessions (Union[Unset, List['NewCampaignSession']]):
        stop_messages (Union[Unset, List['NewMessage']]):
        user_groups (Union[Unset, List['CampaignUserGroup']]):
        welcome_messages (Union[Unset, List['NewMessage']]):
    """

    label: str
    email_notifications: Union[Unset, "EmailNotifications"] = UNSET
    overview_messages: Union[Unset, List["NewMessage"]] = UNSET
    session_modes: Union[Unset, List["NewCampaignSessionMode"]] = UNSET
    sessions: Union[Unset, List["NewCampaignSession"]] = UNSET
    stop_messages: Union[Unset, List["NewMessage"]] = UNSET
    user_groups: Union[Unset, List["CampaignUserGroup"]] = UNSET
    welcome_messages: Union[Unset, List["NewMessage"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        email_notifications: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.email_notifications, Unset):
            email_notifications = self.email_notifications.to_dict()

        overview_messages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.overview_messages, Unset):
            overview_messages = []
            for overview_messages_item_data in self.overview_messages:
                overview_messages_item = overview_messages_item_data.to_dict()

                overview_messages.append(overview_messages_item)

        session_modes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.session_modes, Unset):
            session_modes = []
            for session_modes_item_data in self.session_modes:
                session_modes_item = session_modes_item_data.to_dict()

                session_modes.append(session_modes_item)

        sessions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sessions, Unset):
            sessions = []
            for sessions_item_data in self.sessions:
                sessions_item = sessions_item_data.to_dict()

                sessions.append(sessions_item)

        stop_messages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.stop_messages, Unset):
            stop_messages = []
            for stop_messages_item_data in self.stop_messages:
                stop_messages_item = stop_messages_item_data.to_dict()

                stop_messages.append(stop_messages_item)

        user_groups: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.user_groups, Unset):
            user_groups = []
            for user_groups_item_data in self.user_groups:
                user_groups_item = user_groups_item_data.to_dict()

                user_groups.append(user_groups_item)

        welcome_messages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.welcome_messages, Unset):
            welcome_messages = []
            for welcome_messages_item_data in self.welcome_messages:
                welcome_messages_item = welcome_messages_item_data.to_dict()

                welcome_messages.append(welcome_messages_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "label": label,
            }
        )
        if email_notifications is not UNSET:
            field_dict["emailNotifications"] = email_notifications
        if overview_messages is not UNSET:
            field_dict["overviewMessages"] = overview_messages
        if session_modes is not UNSET:
            field_dict["sessionModes"] = session_modes
        if sessions is not UNSET:
            field_dict["sessions"] = sessions
        if stop_messages is not UNSET:
            field_dict["stopMessages"] = stop_messages
        if user_groups is not UNSET:
            field_dict["userGroups"] = user_groups
        if welcome_messages is not UNSET:
            field_dict["welcomeMessages"] = welcome_messages

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.campaign_user_group import CampaignUserGroup
        from ..models.email_notifications import EmailNotifications
        from ..models.new_campaign_session import NewCampaignSession
        from ..models.new_campaign_session_mode import NewCampaignSessionMode
        from ..models.new_message import NewMessage

        d = src_dict.copy()
        label = d.pop("label")

        _email_notifications = d.pop("emailNotifications", UNSET)
        email_notifications: Union[Unset, EmailNotifications]
        if isinstance(_email_notifications, Unset):
            email_notifications = UNSET
        else:
            email_notifications = EmailNotifications.from_dict(_email_notifications)

        overview_messages = []
        _overview_messages = d.pop("overviewMessages", UNSET)
        for overview_messages_item_data in _overview_messages or []:
            overview_messages_item = NewMessage.from_dict(overview_messages_item_data)

            overview_messages.append(overview_messages_item)

        session_modes = []
        _session_modes = d.pop("sessionModes", UNSET)
        for session_modes_item_data in _session_modes or []:
            session_modes_item = NewCampaignSessionMode.from_dict(session_modes_item_data)

            session_modes.append(session_modes_item)

        sessions = []
        _sessions = d.pop("sessions", UNSET)
        for sessions_item_data in _sessions or []:
            sessions_item = NewCampaignSession.from_dict(sessions_item_data)

            sessions.append(sessions_item)

        stop_messages = []
        _stop_messages = d.pop("stopMessages", UNSET)
        for stop_messages_item_data in _stop_messages or []:
            stop_messages_item = NewMessage.from_dict(stop_messages_item_data)

            stop_messages.append(stop_messages_item)

        user_groups = []
        _user_groups = d.pop("userGroups", UNSET)
        for user_groups_item_data in _user_groups or []:
            user_groups_item = CampaignUserGroup.from_dict(user_groups_item_data)

            user_groups.append(user_groups_item)

        welcome_messages = []
        _welcome_messages = d.pop("welcomeMessages", UNSET)
        for welcome_messages_item_data in _welcome_messages or []:
            welcome_messages_item = NewMessage.from_dict(welcome_messages_item_data)

            welcome_messages.append(welcome_messages_item)

        new_campaign = cls(
            label=label,
            email_notifications=email_notifications,
            overview_messages=overview_messages,
            session_modes=session_modes,
            sessions=sessions,
            stop_messages=stop_messages,
            user_groups=user_groups,
            welcome_messages=welcome_messages,
        )

        return new_campaign



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.model_metrics import ModelMetrics


T = TypeVar("T", bound="ModelsMetrics")


@attr.s(auto_attribs=True)
class ModelsMetrics:
    """
    Attributes:
        history (List['ModelMetrics']):
        best (Union[Unset, ModelMetrics]):
        last (Union[Unset, ModelMetrics]):
    """

    history: List["ModelMetrics"]
    best: Union[Unset, "ModelMetrics"] = UNSET
    last: Union[Unset, "ModelMetrics"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        history = []
        for history_item_data in self.history:
            history_item = history_item_data.to_dict()

            history.append(history_item)

        best: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.best, Unset):
            best = self.best.to_dict()

        last: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.last, Unset):
            last = self.last.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "history": history,
            }
        )
        if best is not UNSET:
            field_dict["best"] = best
        if last is not UNSET:
            field_dict["last"] = last

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.model_metrics import ModelMetrics

        d = src_dict.copy()
        history = []
        _history = d.pop("history")
        for history_item_data in _history:
            history_item = ModelMetrics.from_dict(history_item_data)

            history.append(history_item)

        _best = d.pop("best", UNSET)
        best: Union[Unset, ModelMetrics]
        if isinstance(_best, Unset):
            best = UNSET
        else:
            best = ModelMetrics.from_dict(_best)

        _last = d.pop("last", UNSET)
        last: Union[Unset, ModelMetrics]
        if isinstance(_last, Unset):
            last = UNSET
        else:
            last = ModelMetrics.from_dict(_last)

        models_metrics = cls(
            history=history,
            best=best,
            last=last,
        )

        return models_metrics



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.categories_facets import CategoriesFacets
    from ..models.document_facets import DocumentFacets


T = TypeVar("T", bound="CategoryMetrics")


@attr.s(auto_attribs=True)
class CategoryMetrics:
    """
    Attributes:
        categories_count (int):
        categories_facets (CategoriesFacets):
        document_facets (DocumentFacets):
        documents_in_dataset (int):
    """

    categories_count: int
    categories_facets: "CategoriesFacets"
    document_facets: "DocumentFacets"
    documents_in_dataset: int

    def to_dict(self) -> Dict[str, Any]:
        categories_count = self.categories_count
        categories_facets = self.categories_facets.to_dict()

        document_facets = self.document_facets.to_dict()

        documents_in_dataset = self.documents_in_dataset

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "categoriesCount": categories_count,
                "categoriesFacets": categories_facets,
                "documentFacets": document_facets,
                "documentsInDataset": documents_in_dataset,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.categories_facets import CategoriesFacets
        from ..models.document_facets import DocumentFacets

        d = src_dict.copy()
        categories_count = d.pop("categoriesCount")

        categories_facets = CategoriesFacets.from_dict(d.pop("categoriesFacets"))

        document_facets = DocumentFacets.from_dict(d.pop("documentFacets"))

        documents_in_dataset = d.pop("documentsInDataset")

        category_metrics = cls(
            categories_count=categories_count,
            categories_facets=categories_facets,
            document_facets=document_facets,
            documents_in_dataset=documents_in_dataset,
        )

        return category_metrics



""" Contains all the data models used in inputs/outputs """

from .ack import Ack
from .aggregation import Aggregation
from .annotate_binary_form import AnnotateBinaryForm
from .annotate_binary_with_plan_ref_multipart_data import AnnotateBinaryWithPlanRefMultipartData
from .annotate_documents_with_many import AnnotateDocumentsWithMany
from .annotate_format_binary_with_plan_ref_multipart_data import AnnotateFormatBinaryWithPlanRefMultipartData
from .annotate_text_with_many import AnnotateTextWithMany
from .annotated_doc_annotation import AnnotatedDocAnnotation
from .annotated_doc_annotation_creation_mode import AnnotatedDocAnnotationCreationMode
from .annotated_doc_annotation_properties import AnnotatedDocAnnotationProperties
from .annotated_doc_category import AnnotatedDocCategory
from .annotated_doc_category_creation_mode import AnnotatedDocCategoryCreationMode
from .annotated_doc_category_properties import AnnotatedDocCategoryProperties
from .annotated_document import AnnotatedDocument
from .annotated_document_metadata import AnnotatedDocumentMetadata
from .annotation import Annotation
from .annotation_creation_mode import AnnotationCreationMode
from .annotation_facets import AnnotationFacets
from .annotation_id import AnnotationId
from .annotation_metrics import AnnotationMetrics
from .annotation_patch import AnnotationPatch
from .annotation_patch_status import AnnotationPatchStatus
from .annotation_plan import AnnotationPlan
from .annotation_status import AnnotationStatus
from .annotation_term import AnnotationTerm
from .annotation_term_properties import AnnotationTermProperties
from .annotator import Annotator
from .annotator_multimap import AnnotatorMultimap
from .apply_to import ApplyTo
from .bearer_token import BearerToken
from .bucket import Bucket
from .campaign import Campaign
from .campaign_id import CampaignId
from .campaign_patch import CampaignPatch
from .campaign_roles_change import CampaignRolesChange
from .campaign_session import CampaignSession
from .campaign_session_mode import CampaignSessionMode
from .campaign_user_group import CampaignUserGroup
from .campaign_user_session import CampaignUserSession
from .campaign_user_session_state import CampaignUserSessionState
from .categories_facets import CategoriesFacets
from .category import Category
from .category_action import CategoryAction
from .category_creation_mode import CategoryCreationMode
from .category_id import CategoryId
from .category_metrics import CategoryMetrics
from .category_patch import CategoryPatch
from .category_patch_status import CategoryPatchStatus
from .category_status import CategoryStatus
from .classification_config import ClassificationConfig
from .classification_options import ClassificationOptions
from .config_patch_options import ConfigPatchOptions
from .convert_annotation_plan import ConvertAnnotationPlan
from .convert_format_annotation_plan import ConvertFormatAnnotationPlan
from .converter import Converter
from .converter_parameters import ConverterParameters
from .corpus_metrics import CorpusMetrics
from .create_lexicon_response_200 import CreateLexiconResponse200
from .create_project_from_archive_multipart_data import CreateProjectFromArchiveMultipartData
from .create_term_response_200 import CreateTermResponse200
from .created_by_count import CreatedByCount
from .creation_mode import CreationMode
from .credentials import Credentials
from .delete_group_result import DeleteGroupResult
from .delete_response import DeleteResponse
from .depends_on import DependsOn
from .doc_alt_text import DocAltText
from .doc_annotation import DocAnnotation
from .doc_annotation_creation_mode import DocAnnotationCreationMode
from .doc_annotation_status import DocAnnotationStatus
from .doc_category import DocCategory
from .doc_category_creation_mode import DocCategoryCreationMode
from .doc_category_status import DocCategoryStatus
from .doc_sentence import DocSentence
from .doc_sentence_metadata import DocSentenceMetadata
from .document import Document
from .document_facets import DocumentFacets
from .document_hit import DocumentHit
from .document_hits import DocumentHits
from .document_metadata import DocumentMetadata
from .email_notifications import EmailNotifications
from .engine_config import EngineConfig
from .engine_config_import_summary import EngineConfigImportSummary
from .engine_name import EngineName
from .error import Error
from .experiment import Experiment
from .experiment_parameters import ExperimentParameters
from .experiment_patch import ExperimentPatch
from .experiment_patch_parameters import ExperimentPatchParameters
from .format_binary_form import FormatBinaryForm
from .format_documents_with_many import FormatDocumentsWithMany
from .format_text_with_many import FormatTextWithMany
from .formatter import Formatter
from .formatter_parameters import FormatterParameters
from .gazetteer import Gazetteer
from .gazetteer_parameters import GazetteerParameters
from .gazetteer_patch import GazetteerPatch
from .gazetteer_patch_parameters import GazetteerPatchParameters
from .generated_label_hint import GeneratedLabelHint
from .get_app_state_response_200 import GetAppStateResponse200
from .get_engine_parameters_schema_response_200 import GetEngineParametersSchemaResponse200
from .get_generated_label_names_response_200_item import GetGeneratedLabelNamesResponse200Item
from .get_project_engine_parameters_schema_response_200 import GetProjectEngineParametersSchemaResponse200
from .get_project_messages_scopes_item import GetProjectMessagesScopesItem
from .get_segment_context_context_output import GetSegmentContextContextOutput
from .get_services_distinct_values_response_200_item import GetServicesDistinctValuesResponse200Item
from .get_suggestions_response_200 import GetSuggestionsResponse200
from .group_desc import GroupDesc
from .group_name import GroupName
from .group_patch import GroupPatch
from .group_share import GroupShare
from .http_service_metadata import HttpServiceMetadata
from .http_service_metadata_operations import HttpServiceMetadataOperations
from .http_service_record import HttpServiceRecord
from .import_archive_multipart_data import ImportArchiveMultipartData
from .import_models_multipart_data import ImportModelsMultipartData
from .input_document import InputDocument
from .input_document_metadata import InputDocumentMetadata
from .input_label import InputLabel
from .job_status import JobStatus
from .job_type import JobType
from .label import Label
from .label_count import LabelCount
from .label_names import LabelNames
from .label_set import LabelSet
from .label_set_update import LabelSetUpdate
from .label_update import LabelUpdate
from .launch_document_import_multipart_data import LaunchDocumentImportMultipartData
from .launch_document_import_segmentation_policy import LaunchDocumentImportSegmentationPolicy
from .launch_project_restoration_from_backup_multipart_data import LaunchProjectRestorationFromBackupMultipartData
from .launch_uploaded_document_import_segmentation_policy import LaunchUploadedDocumentImportSegmentationPolicy
from .lexicon import Lexicon
from .lexicon_update import LexiconUpdate
from .localized_message import LocalizedMessage
from .maybe_create_projects_and_import_models_from_archive_multipart_data import (
    MaybeCreateProjectsAndImportModelsFromArchiveMultipartData,
)
from .message import Message
from .message_id import MessageId
from .message_localized import MessageLocalized
from .message_mark import MessageMark
from .message_patch import MessagePatch
from .message_patch_localized import MessagePatchLocalized
from .metadata_count import MetadataCount
from .metadata_definition import MetadataDefinition
from .metadata_definition_entry import MetadataDefinitionEntry
from .model_metrics import ModelMetrics
from .model_metrics_options import ModelMetricsOptions
from .models_metrics import ModelsMetrics
from .named_annotation_plan import NamedAnnotationPlan
from .new_campaign import NewCampaign
from .new_campaign_session import NewCampaignSession
from .new_campaign_session_mode import NewCampaignSessionMode
from .new_campaign_user_session import NewCampaignUserSession
from .new_experiment import NewExperiment
from .new_experiment_parameters import NewExperimentParameters
from .new_gazetteer import NewGazetteer
from .new_gazetteer_parameters import NewGazetteerParameters
from .new_group_desc import NewGroupDesc
from .new_message import NewMessage
from .new_message_localized import NewMessageLocalized
from .new_named_annotation_plan import NewNamedAnnotationPlan
from .new_role import NewRole
from .new_suggester import NewSuggester
from .new_suggester_parameters import NewSuggesterParameters
from .new_user import NewUser
from .next_step import NextStep
from .operation_count import OperationCount
from .partial_label import PartialLabel
from .partial_lexicon import PartialLexicon
from .plan_patch import PlanPatch
from .project_annotators import ProjectAnnotators
from .project_bean import ProjectBean
from .project_config_creation import ProjectConfigCreation
from .project_config_creation_properties import ProjectConfigCreationProperties
from .project_open_session import ProjectOpenSession
from .project_open_session_state import ProjectOpenSessionState
from .project_property import ProjectProperty
from .project_status import ProjectStatus
from .projects_annotators import ProjectsAnnotators
from .quality_figures import QualityFigures
from .relation import Relation
from .report import Report
from .report_classes import ReportClasses
from .request_jwt_token_project_access_mode import RequestJwtTokenProjectAccessMode
from .role_desc import RoleDesc
from .role_update import RoleUpdate
from .search_total import SearchTotal
from .search_total_relation import SearchTotalRelation
from .segment import Segment
from .segment_context import SegmentContext
from .segment_contexts import SegmentContexts
from .segment_hit import SegmentHit
from .segment_hits import SegmentHits
from .segment_metadata import SegmentMetadata
from .session_corpus_permission_change import SessionCorpusPermissionChange
from .share_mode import ShareMode
from .sherpa_job_bean import SherpaJobBean
from .sherpa_job_bean_status import SherpaJobBeanStatus
from .sherpa_job_bean_type import SherpaJobBeanType
from .simple_metadata import SimpleMetadata
from .suggester import Suggester
from .suggester_parameters import SuggesterParameters
from .suggester_patch import SuggesterPatch
from .suggester_patch_parameters import SuggesterPatchParameters
from .suggestion_facets import SuggestionFacets
from .term import Term
from .term_hit import TermHit
from .term_hit_term import TermHitTerm
from .term_hits import TermHits
from .term_import import TermImport
from .term_importer_spec import TermImporterSpec
from .term_importer_spec_parameters import TermImporterSpecParameters
from .text_count import TextCount
from .upload_files_multipart_data import UploadFilesMultipartData
from .uploaded_file import UploadedFile
from .uploaded_file_info import UploadedFileInfo
from .user_permissions_update import UserPermissionsUpdate
from .user_profile import UserProfile
from .user_profile_update import UserProfileUpdate
from .user_response import UserResponse
from .user_session_action_action import UserSessionActionAction
from .user_session_state import UserSessionState
from .with_annotator import WithAnnotator
from .with_annotator_condition import WithAnnotatorCondition
from .with_annotator_parameters import WithAnnotatorParameters
from .with_processor import WithProcessor
from .with_processor_condition import WithProcessorCondition
from .with_processor_parameters import WithProcessorParameters
from .with_sentencizer import WithSentencizer
from .with_sentencizer_condition import WithSentencizerCondition
from .with_sentencizer_parameters import WithSentencizerParameters

__all__ = (
    "Ack",
    "Aggregation",
    "AnnotateBinaryForm",
    "AnnotateBinaryWithPlanRefMultipartData",
    "AnnotatedDocAnnotation",
    "AnnotatedDocAnnotationCreationMode",
    "AnnotatedDocAnnotationProperties",
    "AnnotatedDocCategory",
    "AnnotatedDocCategoryCreationMode",
    "AnnotatedDocCategoryProperties",
    "AnnotatedDocument",
    "AnnotatedDocumentMetadata",
    "AnnotateDocumentsWithMany",
    "AnnotateFormatBinaryWithPlanRefMultipartData",
    "AnnotateTextWithMany",
    "Annotation",
    "AnnotationCreationMode",
    "AnnotationFacets",
    "AnnotationId",
    "AnnotationMetrics",
    "AnnotationPatch",
    "AnnotationPatchStatus",
    "AnnotationPlan",
    "AnnotationStatus",
    "AnnotationTerm",
    "AnnotationTermProperties",
    "Annotator",
    "AnnotatorMultimap",
    "ApplyTo",
    "BearerToken",
    "Bucket",
    "Campaign",
    "CampaignId",
    "CampaignPatch",
    "CampaignRolesChange",
    "CampaignSession",
    "CampaignSessionMode",
    "CampaignUserGroup",
    "CampaignUserSession",
    "CampaignUserSessionState",
    "CategoriesFacets",
    "Category",
    "CategoryAction",
    "CategoryCreationMode",
    "CategoryId",
    "CategoryMetrics",
    "CategoryPatch",
    "CategoryPatchStatus",
    "CategoryStatus",
    "ClassificationConfig",
    "ClassificationOptions",
    "ConfigPatchOptions",
    "ConvertAnnotationPlan",
    "Converter",
    "ConverterParameters",
    "ConvertFormatAnnotationPlan",
    "CorpusMetrics",
    "CreatedByCount",
    "CreateLexiconResponse200",
    "CreateProjectFromArchiveMultipartData",
    "CreateTermResponse200",
    "CreationMode",
    "Credentials",
    "DeleteGroupResult",
    "DeleteResponse",
    "DependsOn",
    "DocAltText",
    "DocAnnotation",
    "DocAnnotationCreationMode",
    "DocAnnotationStatus",
    "DocCategory",
    "DocCategoryCreationMode",
    "DocCategoryStatus",
    "DocSentence",
    "DocSentenceMetadata",
    "Document",
    "DocumentFacets",
    "DocumentHit",
    "DocumentHits",
    "DocumentMetadata",
    "EmailNotifications",
    "EngineConfig",
    "EngineConfigImportSummary",
    "EngineName",
    "Error",
    "Experiment",
    "ExperimentParameters",
    "ExperimentPatch",
    "ExperimentPatchParameters",
    "FormatBinaryForm",
    "FormatDocumentsWithMany",
    "Formatter",
    "FormatterParameters",
    "FormatTextWithMany",
    "Gazetteer",
    "GazetteerParameters",
    "GazetteerPatch",
    "GazetteerPatchParameters",
    "GeneratedLabelHint",
    "GetAppStateResponse200",
    "GetEngineParametersSchemaResponse200",
    "GetGeneratedLabelNamesResponse200Item",
    "GetProjectEngineParametersSchemaResponse200",
    "GetProjectMessagesScopesItem",
    "GetSegmentContextContextOutput",
    "GetServicesDistinctValuesResponse200Item",
    "GetSuggestionsResponse200",
    "GroupDesc",
    "GroupName",
    "GroupPatch",
    "GroupShare",
    "HttpServiceMetadata",
    "HttpServiceMetadataOperations",
    "HttpServiceRecord",
    "ImportArchiveMultipartData",
    "ImportModelsMultipartData",
    "InputDocument",
    "InputDocumentMetadata",
    "InputLabel",
    "JobStatus",
    "JobType",
    "Label",
    "LabelCount",
    "LabelNames",
    "LabelSet",
    "LabelSetUpdate",
    "LabelUpdate",
    "LaunchDocumentImportMultipartData",
    "LaunchDocumentImportSegmentationPolicy",
    "LaunchProjectRestorationFromBackupMultipartData",
    "LaunchUploadedDocumentImportSegmentationPolicy",
    "Lexicon",
    "LexiconUpdate",
    "LocalizedMessage",
    "MaybeCreateProjectsAndImportModelsFromArchiveMultipartData",
    "Message",
    "MessageId",
    "MessageLocalized",
    "MessageMark",
    "MessagePatch",
    "MessagePatchLocalized",
    "MetadataCount",
    "MetadataDefinition",
    "MetadataDefinitionEntry",
    "ModelMetrics",
    "ModelMetricsOptions",
    "ModelsMetrics",
    "NamedAnnotationPlan",
    "NewCampaign",
    "NewCampaignSession",
    "NewCampaignSessionMode",
    "NewCampaignUserSession",
    "NewExperiment",
    "NewExperimentParameters",
    "NewGazetteer",
    "NewGazetteerParameters",
    "NewGroupDesc",
    "NewMessage",
    "NewMessageLocalized",
    "NewNamedAnnotationPlan",
    "NewRole",
    "NewSuggester",
    "NewSuggesterParameters",
    "NewUser",
    "NextStep",
    "OperationCount",
    "PartialLabel",
    "PartialLexicon",
    "PlanPatch",
    "ProjectAnnotators",
    "ProjectBean",
    "ProjectConfigCreation",
    "ProjectConfigCreationProperties",
    "ProjectOpenSession",
    "ProjectOpenSessionState",
    "ProjectProperty",
    "ProjectsAnnotators",
    "ProjectStatus",
    "QualityFigures",
    "Relation",
    "Report",
    "ReportClasses",
    "RequestJwtTokenProjectAccessMode",
    "RoleDesc",
    "RoleUpdate",
    "SearchTotal",
    "SearchTotalRelation",
    "Segment",
    "SegmentContext",
    "SegmentContexts",
    "SegmentHit",
    "SegmentHits",
    "SegmentMetadata",
    "SessionCorpusPermissionChange",
    "ShareMode",
    "SherpaJobBean",
    "SherpaJobBeanStatus",
    "SherpaJobBeanType",
    "SimpleMetadata",
    "Suggester",
    "SuggesterParameters",
    "SuggesterPatch",
    "SuggesterPatchParameters",
    "SuggestionFacets",
    "Term",
    "TermHit",
    "TermHits",
    "TermHitTerm",
    "TermImport",
    "TermImporterSpec",
    "TermImporterSpecParameters",
    "TextCount",
    "UploadedFile",
    "UploadedFileInfo",
    "UploadFilesMultipartData",
    "UserPermissionsUpdate",
    "UserProfile",
    "UserProfileUpdate",
    "UserResponse",
    "UserSessionActionAction",
    "UserSessionState",
    "WithAnnotator",
    "WithAnnotatorCondition",
    "WithAnnotatorParameters",
    "WithProcessor",
    "WithProcessorCondition",
    "WithProcessorParameters",
    "WithSentencizer",
    "WithSentencizerCondition",
    "WithSentencizerParameters",
)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CampaignSessionMode")


@attr.s(auto_attribs=True)
class CampaignSessionMode:
    """
    Attributes:
        id (str):
        label (str):
        open_functional_roles (Union[Unset, List[str]]):
        started_functional_roles (Union[Unset, List[str]]):
    """

    id: str
    label: str
    open_functional_roles: Union[Unset, List[str]] = UNSET
    started_functional_roles: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        label = self.label
        open_functional_roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.open_functional_roles, Unset):
            open_functional_roles = self.open_functional_roles

        started_functional_roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.started_functional_roles, Unset):
            started_functional_roles = self.started_functional_roles

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "label": label,
            }
        )
        if open_functional_roles is not UNSET:
            field_dict["openFunctionalRoles"] = open_functional_roles
        if started_functional_roles is not UNSET:
            field_dict["startedFunctionalRoles"] = started_functional_roles

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        label = d.pop("label")

        open_functional_roles = cast(List[str], d.pop("openFunctionalRoles", UNSET))

        started_functional_roles = cast(List[str], d.pop("startedFunctionalRoles", UNSET))

        campaign_session_mode = cls(
            id=id,
            label=label,
            open_functional_roles=open_functional_roles,
            started_functional_roles=started_functional_roles,
        )

        return campaign_session_mode



from enum import Enum


class CampaignUserSessionState(str, Enum):
    NOT_EXISTS = "NOT_EXISTS"
    NEW = "NEW"
    OPEN = "OPEN"
    STARTED = "STARTED"
    PAUSED = "PAUSED"
    STOPPED = "STOPPED"
    CLOSED = "CLOSED"
    SAME = "SAME"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GetProjectMessagesScopesItem(str, Enum):
    OPEN_CAMPAIGN = "open_campaign"
    OPEN_SESSION = "open_session"
    STOP_CAMPAIGN = "stop_campaign"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.category_patch_status import CategoryPatchStatus

T = TypeVar("T", bound="CategoryPatch")


@attr.s(auto_attribs=True)
class CategoryPatch:
    """
    Attributes:
        status (CategoryPatchStatus): Status of the category
    """

    status: CategoryPatchStatus

    def to_dict(self) -> Dict[str, Any]:
        status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "status": status,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = CategoryPatchStatus(d.pop("status"))

        category_patch = cls(
            status=status,
        )

        return category_patch



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="GeneratedLabelHint")


@attr.s(auto_attribs=True)
class GeneratedLabelHint:
    """
    Attributes:
        label_hint (str):
    """

    label_hint: str

    def to_dict(self) -> Dict[str, Any]:
        label_hint = self.label_hint

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "labelHint": label_hint,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label_hint = d.pop("labelHint")

        generated_label_hint = cls(
            label_hint=label_hint,
        )

        return generated_label_hint



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="SessionCorpusPermissionChange")


@attr.s(auto_attribs=True)
class SessionCorpusPermissionChange:
    """
    Attributes:
        add (bool):
        project_name (str):
        session_label (str):
        usernames (List[Any]):
    """

    add: bool
    project_name: str
    session_label: str
    usernames: List[Any]

    def to_dict(self) -> Dict[str, Any]:
        add = self.add
        project_name = self.project_name
        session_label = self.session_label
        usernames = self.usernames

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "add": add,
                "projectName": project_name,
                "sessionLabel": session_label,
                "usernames": usernames,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        add = d.pop("add")

        project_name = d.pop("projectName")

        session_label = d.pop("sessionLabel")

        usernames = cast(List[Any], d.pop("usernames"))

        session_corpus_permission_change = cls(
            add=add,
            project_name=project_name,
            session_label=session_label,
            usernames=usernames,
        )

        return session_corpus_permission_change



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="UserSessionState")


@attr.s(auto_attribs=True)
class UserSessionState:
    """
    Attributes:
        enumname (bool):
    """

    enumname: bool

    def to_dict(self) -> Dict[str, Any]:
        enumname = self.enumname

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "$enum$name": enumname,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enumname = d.pop("$enum$name")

        user_session_state = cls(
            enumname=enumname,
        )

        return user_session_state



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.segment import Segment


T = TypeVar("T", bound="SegmentContext")


@attr.s(auto_attribs=True)
class SegmentContext:
    """
    Attributes:
        size (int):
        merged (Union[Unset, Segment]):
        segments (Union[Unset, List['Segment']]):
    """

    size: int
    merged: Union[Unset, "Segment"] = UNSET
    segments: Union[Unset, List["Segment"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        size = self.size
        merged: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.merged, Unset):
            merged = self.merged.to_dict()

        segments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.segments, Unset):
            segments = []
            for segments_item_data in self.segments:
                segments_item = segments_item_data.to_dict()

                segments.append(segments_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "size": size,
            }
        )
        if merged is not UNSET:
            field_dict["merged"] = merged
        if segments is not UNSET:
            field_dict["segments"] = segments

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.segment import Segment

        d = src_dict.copy()
        size = d.pop("size")

        _merged = d.pop("merged", UNSET)
        merged: Union[Unset, Segment]
        if isinstance(_merged, Unset):
            merged = UNSET
        else:
            merged = Segment.from_dict(_merged)

        segments = []
        _segments = d.pop("segments", UNSET)
        for segments_item_data in _segments or []:
            segments_item = Segment.from_dict(segments_item_data)

            segments.append(segments_item)

        segment_context = cls(
            size=size,
            merged=merged,
            segments=segments,
        )

        return segment_context



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.segment_context import SegmentContext


T = TypeVar("T", bound="SegmentContexts")


@attr.s(auto_attribs=True)
class SegmentContexts:
    """
    Attributes:
        after (SegmentContext):
        before (SegmentContext):
    """

    after: "SegmentContext"
    before: "SegmentContext"

    def to_dict(self) -> Dict[str, Any]:
        after = self.after.to_dict()

        before = self.before.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "after": after,
                "before": before,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.segment_context import SegmentContext

        d = src_dict.copy()
        after = SegmentContext.from_dict(d.pop("after"))

        before = SegmentContext.from_dict(d.pop("before"))

        segment_contexts = cls(
            after=after,
            before=before,
        )

        return segment_contexts



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.annotator import Annotator


T = TypeVar("T", bound="ProjectAnnotators")


@attr.s(auto_attribs=True)
class ProjectAnnotators:
    """
    Attributes:
        gazetteer (List['Annotator']):
        learner (List['Annotator']):
        plan (List['Annotator']):
        suggester (List['Annotator']):
    """

    gazetteer: List["Annotator"]
    learner: List["Annotator"]
    plan: List["Annotator"]
    suggester: List["Annotator"]

    def to_dict(self) -> Dict[str, Any]:
        gazetteer = []
        for gazetteer_item_data in self.gazetteer:
            gazetteer_item = gazetteer_item_data.to_dict()

            gazetteer.append(gazetteer_item)

        learner = []
        for learner_item_data in self.learner:
            learner_item = learner_item_data.to_dict()

            learner.append(learner_item)

        plan = []
        for plan_item_data in self.plan:
            plan_item = plan_item_data.to_dict()

            plan.append(plan_item)

        suggester = []
        for suggester_item_data in self.suggester:
            suggester_item = suggester_item_data.to_dict()

            suggester.append(suggester_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "gazetteer": gazetteer,
                "learner": learner,
                "plan": plan,
                "suggester": suggester,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotator import Annotator

        d = src_dict.copy()
        gazetteer = []
        _gazetteer = d.pop("gazetteer")
        for gazetteer_item_data in _gazetteer:
            gazetteer_item = Annotator.from_dict(gazetteer_item_data)

            gazetteer.append(gazetteer_item)

        learner = []
        _learner = d.pop("learner")
        for learner_item_data in _learner:
            learner_item = Annotator.from_dict(learner_item_data)

            learner.append(learner_item)

        plan = []
        _plan = d.pop("plan")
        for plan_item_data in _plan:
            plan_item = Annotator.from_dict(plan_item_data)

            plan.append(plan_item)

        suggester = []
        _suggester = d.pop("suggester")
        for suggester_item_data in _suggester:
            suggester_item = Annotator.from_dict(suggester_item_data)

            suggester.append(suggester_item)

        project_annotators = cls(
            gazetteer=gazetteer,
            learner=learner,
            plan=plan,
            suggester=suggester,
        )

        return project_annotators
